#
# This file is part of pyasn1-modules software.
#
# Created by Russ Housley.
#
# Copyright (c) 2019, Vigil Security, LLC
# License: https://www.pysnmp.com/pyasn1/license.html
#
# NSA's CMS Key Management Attributes
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc7906.txt
# https://www.rfc-editor.org/errata/eid5850
#

from pyasn1.type import char, constraint, namedtype, namedval, tag, univ

from pyasn1_modules import rfc2634, rfc4108, rfc5280, rfc5652, rfc6010, rfc6019, rfc7191

MAX = float("inf")


# Imports From RFC 2634

id_aa_contentHint = rfc2634.id_aa_contentHint

ContentHints = rfc2634.ContentHints

id_aa_securityLabel = rfc2634.id_aa_securityLabel

SecurityPolicyIdentifier = rfc2634.SecurityPolicyIdentifier

SecurityClassification = rfc2634.SecurityClassification

ESSPrivacyMark = rfc2634.ESSPrivacyMark

SecurityCategories = rfc2634.SecurityCategories

ESSSecurityLabel = rfc2634.ESSSecurityLabel


# Imports From RFC 4108

id_aa_communityIdentifiers = rfc4108.id_aa_communityIdentifiers

CommunityIdentifier = rfc4108.CommunityIdentifier

CommunityIdentifiers = rfc4108.CommunityIdentifiers


# Imports From RFC 5280

AlgorithmIdentifier = rfc5280.AlgorithmIdentifier

Name = rfc5280.Name

Certificate = rfc5280.Certificate

GeneralNames = rfc5280.GeneralNames

GeneralName = rfc5280.GeneralName


SubjectInfoAccessSyntax = rfc5280.SubjectInfoAccessSyntax

id_pkix = rfc5280.id_pkix

id_pe = rfc5280.id_pe

id_pe_subjectInfoAccess = rfc5280.id_pe_subjectInfoAccess


# Imports From RFC 6010

CMSContentConstraints = rfc6010.CMSContentConstraints


# Imports From RFC 6019

BinaryTime = rfc6019.BinaryTime

id_aa_binarySigningTime = rfc6019.id_aa_binarySigningTime

BinarySigningTime = rfc6019.BinarySigningTime


# Imports From RFC 5652

Attribute = rfc5652.Attribute

CertificateSet = rfc5652.CertificateSet

CertificateChoices = rfc5652.CertificateChoices

id_contentType = rfc5652.id_contentType

ContentType = rfc5652.ContentType

id_messageDigest = rfc5652.id_messageDigest

MessageDigest = rfc5652.MessageDigest


# Imports From RFC 7191

SIREntityName = rfc7191.SIREntityName

id_aa_KP_keyPkgIdAndReceiptReq = rfc7191.id_aa_KP_keyPkgIdAndReceiptReq

KeyPkgIdentifierAndReceiptReq = rfc7191.KeyPkgIdentifierAndReceiptReq


# Key Province Attribute

id_aa_KP_keyProvinceV2 = univ.ObjectIdentifier("2.16.840.1.101.2.1.5.71")


class KeyProvinceV2(univ.ObjectIdentifier):
    pass


aa_keyProvince_v2 = Attribute()
aa_keyProvince_v2["attrType"] = id_aa_KP_keyProvinceV2
aa_keyProvince_v2["attrValues"][0] = KeyProvinceV2()


# Manifest Attribute

id_aa_KP_manifest = univ.ObjectIdentifier("2.16.840.1.101.2.1.5.72")


class ShortTitle(char.PrintableString):
    pass


class Manifest(univ.SequenceOf):
    pass


Manifest.componentType = ShortTitle()
Manifest.subtypeSpec = constraint.ValueSizeConstraint(1, MAX)


aa_manifest = Attribute()
aa_manifest["attrType"] = id_aa_KP_manifest
aa_manifest["attrValues"][0] = Manifest()


# Key Algorithm Attribute

id_kma_keyAlgorithm = univ.ObjectIdentifier("2.16.840.1.101.2.1.13.1")


class KeyAlgorithm(univ.Sequence):
    pass


KeyAlgorithm.componentType = namedtype.NamedTypes(
    namedtype.NamedType("keyAlg", univ.ObjectIdentifier()),
    namedtype.OptionalNamedType(
        "checkWordAlg",
        univ.ObjectIdentifier().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
        ),
    ),
    namedtype.OptionalNamedType(
        "crcAlg",
        univ.ObjectIdentifier().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 2)
        ),
    ),
)


aa_keyAlgorithm = Attribute()
aa_keyAlgorithm["attrType"] = id_kma_keyAlgorithm
aa_keyAlgorithm["attrValues"][0] = KeyAlgorithm()


# User Certificate Attribute

id_at_userCertificate = univ.ObjectIdentifier("2.5.4.36")


aa_userCertificate = Attribute()
aa_userCertificate["attrType"] = id_at_userCertificate
aa_userCertificate["attrValues"][0] = Certificate()


# Key Package Receivers Attribute

id_kma_keyPkgReceiversV2 = univ.ObjectIdentifier("2.16.840.1.101.2.1.13.16")


class KeyPkgReceiver(univ.Choice):
    pass


KeyPkgReceiver.componentType = namedtype.NamedTypes(
    namedtype.NamedType(
        "sirEntity",
        SIREntityName().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
        ),
    ),
    namedtype.NamedType(
        "community",
        CommunityIdentifier().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
        ),
    ),
)


class KeyPkgReceiversV2(univ.SequenceOf):
    pass


KeyPkgReceiversV2.componentType = KeyPkgReceiver()
KeyPkgReceiversV2.subtypeSpec = constraint.ValueSizeConstraint(1, MAX)


aa_keyPackageReceivers_v2 = Attribute()
aa_keyPackageReceivers_v2["attrType"] = id_kma_keyPkgReceiversV2
aa_keyPackageReceivers_v2["attrValues"][0] = KeyPkgReceiversV2()


# TSEC Nomenclature Attribute

id_kma_TSECNomenclature = univ.ObjectIdentifier("2.16.840.1.101.2.1.13.3")


class CharEdition(char.PrintableString):
    pass


class CharEditionRange(univ.Sequence):
    pass


CharEditionRange.componentType = namedtype.NamedTypes(
    namedtype.NamedType("firstCharEdition", CharEdition()),
    namedtype.NamedType("lastCharEdition", CharEdition()),
)


class NumEdition(univ.Integer):
    pass


NumEdition.subtypeSpec = constraint.ValueRangeConstraint(0, 308915776)


class NumEditionRange(univ.Sequence):
    pass


NumEditionRange.componentType = namedtype.NamedTypes(
    namedtype.NamedType("firstNumEdition", NumEdition()),
    namedtype.NamedType("lastNumEdition", NumEdition()),
)


class EditionID(univ.Choice):
    pass


EditionID.componentType = namedtype.NamedTypes(
    namedtype.NamedType(
        "char",
        univ.Choice(
            componentType=namedtype.NamedTypes(
                namedtype.NamedType(
                    "charEdition",
                    CharEdition().subtype(
                        implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
                    ),
                ),
                namedtype.NamedType(
                    "charEditionRange",
                    CharEditionRange().subtype(
                        implicitTag=tag.Tag(
                            tag.tagClassContext, tag.tagFormatConstructed, 2
                        )
                    ),
                ),
            )
        ),
    ),
    namedtype.NamedType(
        "num",
        univ.Choice(
            componentType=namedtype.NamedTypes(
                namedtype.NamedType(
                    "numEdition",
                    NumEdition().subtype(
                        implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 3)
                    ),
                ),
                namedtype.NamedType(
                    "numEditionRange",
                    NumEditionRange().subtype(
                        implicitTag=tag.Tag(
                            tag.tagClassContext, tag.tagFormatConstructed, 4
                        )
                    ),
                ),
            )
        ),
    ),
)


class Register(univ.Integer):
    pass


Register.subtypeSpec = constraint.ValueRangeConstraint(0, 2147483647)


class RegisterRange(univ.Sequence):
    pass


RegisterRange.componentType = namedtype.NamedTypes(
    namedtype.NamedType("firstRegister", Register()),
    namedtype.NamedType("lastRegister", Register()),
)


class RegisterID(univ.Choice):
    pass


RegisterID.componentType = namedtype.NamedTypes(
    namedtype.NamedType(
        "register",
        Register().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 5)
        ),
    ),
    namedtype.NamedType(
        "registerRange",
        RegisterRange().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 6)
        ),
    ),
)


class SegmentNumber(univ.Integer):
    pass


SegmentNumber.subtypeSpec = constraint.ValueRangeConstraint(1, 127)


class SegmentRange(univ.Sequence):
    pass


SegmentRange.componentType = namedtype.NamedTypes(
    namedtype.NamedType("firstSegment", SegmentNumber()),
    namedtype.NamedType("lastSegment", SegmentNumber()),
)


class SegmentID(univ.Choice):
    pass


SegmentID.componentType = namedtype.NamedTypes(
    namedtype.NamedType(
        "segmentNumber",
        SegmentNumber().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 7)
        ),
    ),
    namedtype.NamedType(
        "segmentRange",
        SegmentRange().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 8)
        ),
    ),
)


class TSECNomenclature(univ.Sequence):
    pass


TSECNomenclature.componentType = namedtype.NamedTypes(
    namedtype.NamedType("shortTitle", ShortTitle()),
    namedtype.OptionalNamedType("editionID", EditionID()),
    namedtype.OptionalNamedType("registerID", RegisterID()),
    namedtype.OptionalNamedType("segmentID", SegmentID()),
)


aa_tsecNomenclature = Attribute()
aa_tsecNomenclature["attrType"] = id_kma_TSECNomenclature
aa_tsecNomenclature["attrValues"][0] = TSECNomenclature()


# Key Purpose Attribute

id_kma_keyPurpose = univ.ObjectIdentifier("2.16.840.1.101.2.1.13.13")


class KeyPurpose(univ.Enumerated):
    pass


KeyPurpose.namedValues = namedval.NamedValues(
    ("n-a", 0),
    ("a", 65),
    ("b", 66),
    ("l", 76),
    ("m", 77),
    ("r", 82),
    ("s", 83),
    ("t", 84),
    ("v", 86),
    ("x", 88),
    ("z", 90),
)


aa_keyPurpose = Attribute()
aa_keyPurpose["attrType"] = id_kma_keyPurpose
aa_keyPurpose["attrValues"][0] = KeyPurpose()


# Key Use Attribute

id_kma_keyUse = univ.ObjectIdentifier("2.16.840.1.101.2.1.13.14")


class KeyUse(univ.Enumerated):
    pass


KeyUse.namedValues = namedval.NamedValues(
    ("n-a", 0),
    ("ffk", 1),
    ("kek", 2),
    ("kpk", 3),
    ("msk", 4),
    ("qkek", 5),
    ("tek", 6),
    ("tsk", 7),
    ("trkek", 8),
    ("nfk", 9),
    ("effk", 10),
    ("ebfk", 11),
    ("aek", 12),
    ("wod", 13),
    ("kesk", 246),
    ("eik", 247),
    ("ask", 248),
    ("kmk", 249),
    ("rsk", 250),
    ("csk", 251),
    ("sak", 252),
    ("rgk", 253),
    ("cek", 254),
    ("exk", 255),
)


aa_keyUse = Attribute()
aa_keyPurpose["attrType"] = id_kma_keyUse
aa_keyPurpose["attrValues"][0] = KeyUse()


# Transport Key Attribute

id_kma_transportKey = univ.ObjectIdentifier("2.16.840.1.101.2.1.13.15")


class TransOp(univ.Enumerated):
    pass


TransOp.namedValues = namedval.NamedValues(("transport", 1), ("operational", 2))


aa_transportKey = Attribute()
aa_transportKey["attrType"] = id_kma_transportKey
aa_transportKey["attrValues"][0] = TransOp()


# Key Distribution Period Attribute

id_kma_keyDistPeriod = univ.ObjectIdentifier("2.16.840.1.101.2.1.13.5")


class KeyDistPeriod(univ.Sequence):
    pass


KeyDistPeriod.componentType = namedtype.NamedTypes(
    namedtype.OptionalNamedType(
        "doNotDistBefore",
        BinaryTime().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
        ),
    ),
    namedtype.NamedType("doNotDistAfter", BinaryTime()),
)


aa_keyDistributionPeriod = Attribute()
aa_keyDistributionPeriod["attrType"] = id_kma_keyDistPeriod
aa_keyDistributionPeriod["attrValues"][0] = KeyDistPeriod()


# Key Validity Period Attribute

id_kma_keyValidityPeriod = univ.ObjectIdentifier("2.16.840.1.101.2.1.13.6")


class KeyValidityPeriod(univ.Sequence):
    pass


KeyValidityPeriod.componentType = namedtype.NamedTypes(
    namedtype.NamedType("doNotUseBefore", BinaryTime()),
    namedtype.OptionalNamedType("doNotUseAfter", BinaryTime()),
)


aa_keyValidityPeriod = Attribute()
aa_keyValidityPeriod["attrType"] = id_kma_keyValidityPeriod
aa_keyValidityPeriod["attrValues"][0] = KeyValidityPeriod()


# Key Duration Attribute

id_kma_keyDuration = univ.ObjectIdentifier("2.16.840.1.101.2.1.13.7")


ub_KeyDuration_months = univ.Integer(72)

ub_KeyDuration_hours = univ.Integer(96)

ub_KeyDuration_days = univ.Integer(732)

ub_KeyDuration_weeks = univ.Integer(104)

ub_KeyDuration_years = univ.Integer(100)


class KeyDuration(univ.Choice):
    pass


KeyDuration.componentType = namedtype.NamedTypes(
    namedtype.NamedType(
        "hours",
        univ.Integer()
        .subtype(subtypeSpec=constraint.ValueRangeConstraint(1, ub_KeyDuration_hours))
        .subtype(implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)),
    ),
    namedtype.NamedType(
        "days",
        univ.Integer().subtype(
            subtypeSpec=constraint.ValueRangeConstraint(1, ub_KeyDuration_days)
        ),
    ),
    namedtype.NamedType(
        "weeks",
        univ.Integer()
        .subtype(subtypeSpec=constraint.ValueRangeConstraint(1, ub_KeyDuration_weeks))
        .subtype(implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)),
    ),
    namedtype.NamedType(
        "months",
        univ.Integer()
        .subtype(subtypeSpec=constraint.ValueRangeConstraint(1, ub_KeyDuration_months))
        .subtype(implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 2)),
    ),
    namedtype.NamedType(
        "years",
        univ.Integer()
        .subtype(subtypeSpec=constraint.ValueRangeConstraint(1, ub_KeyDuration_years))
        .subtype(implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 3)),
    ),
)


aa_keyDurationPeriod = Attribute()
aa_keyDurationPeriod["attrType"] = id_kma_keyDuration
aa_keyDurationPeriod["attrValues"][0] = KeyDuration()


# Classification Attribute

id_aa_KP_classification = univ.ObjectIdentifier(id_aa_securityLabel)


id_enumeratedPermissiveAttributes = univ.ObjectIdentifier("2.16.840.1.101.2.1.8.3.1")

id_enumeratedRestrictiveAttributes = univ.ObjectIdentifier("2.16.840.1.101.2.1.8.3.4")

id_informativeAttributes = univ.ObjectIdentifier("2.16.840.1.101.2.1.8.3.3")


class SecurityAttribute(univ.Integer):
    pass


SecurityAttribute.subtypeSpec = constraint.ValueRangeConstraint(0, MAX)


class EnumeratedTag(univ.Sequence):
    pass


EnumeratedTag.componentType = namedtype.NamedTypes(
    namedtype.NamedType("tagName", univ.ObjectIdentifier()),
    namedtype.NamedType("attributeList", univ.SetOf(componentType=SecurityAttribute())),
)


class FreeFormField(univ.Choice):
    pass


FreeFormField.componentType = namedtype.NamedTypes(
    namedtype.NamedType(
        "bitSetAttributes", univ.BitString()
    ),  # Not permitted in RFC 7906
    namedtype.NamedType(
        "securityAttributes", univ.SetOf(componentType=SecurityAttribute())
    ),
)


class InformativeTag(univ.Sequence):
    pass


InformativeTag.componentType = namedtype.NamedTypes(
    namedtype.NamedType("tagName", univ.ObjectIdentifier()),
    namedtype.NamedType("attributes", FreeFormField()),
)


class Classification(ESSSecurityLabel):
    pass


aa_classification = Attribute()
aa_classification["attrType"] = id_aa_KP_classification
aa_classification["attrValues"][0] = Classification()


# Split Identifier Attribute

id_kma_splitID = univ.ObjectIdentifier("2.16.840.1.101.2.1.13.11")


class SplitID(univ.Sequence):
    pass


SplitID.componentType = namedtype.NamedTypes(
    namedtype.NamedType(
        "half", univ.Enumerated(namedValues=namedval.NamedValues(("a", 0), ("b", 1)))
    ),
    namedtype.OptionalNamedType("combineAlg", AlgorithmIdentifier()),
)


aa_splitIdentifier = Attribute()
aa_splitIdentifier["attrType"] = id_kma_splitID
aa_splitIdentifier["attrValues"][0] = SplitID()


# Key Package Type Attribute

id_kma_keyPkgType = univ.ObjectIdentifier("2.16.840.1.101.2.1.13.12")


class KeyPkgType(univ.ObjectIdentifier):
    pass


aa_keyPackageType = Attribute()
aa_keyPackageType["attrType"] = id_kma_keyPkgType
aa_keyPackageType["attrValues"][0] = KeyPkgType()


# Signature Usage Attribute

id_kma_sigUsageV3 = univ.ObjectIdentifier("2.16.840.1.101.2.1.13.22")


class SignatureUsage(CMSContentConstraints):
    pass


aa_signatureUsage_v3 = Attribute()
aa_signatureUsage_v3["attrType"] = id_kma_sigUsageV3
aa_signatureUsage_v3["attrValues"][0] = SignatureUsage()


# Other Certificate Format Attribute

id_kma_otherCertFormats = univ.ObjectIdentifier("2.16.840.1.101.2.1.13.19")


aa_otherCertificateFormats = Attribute()
aa_signatureUsage_v3["attrType"] = id_kma_otherCertFormats
aa_signatureUsage_v3["attrValues"][0] = CertificateChoices()


# PKI Path Attribute

id_at_pkiPath = univ.ObjectIdentifier("2.5.4.70")


class PkiPath(univ.SequenceOf):
    pass


PkiPath.componentType = Certificate()
PkiPath.subtypeSpec = constraint.ValueSizeConstraint(1, MAX)


aa_pkiPath = Attribute()
aa_pkiPath["attrType"] = id_at_pkiPath
aa_pkiPath["attrValues"][0] = PkiPath()


# Useful Certificates Attribute

id_kma_usefulCerts = univ.ObjectIdentifier("2.16.840.1.101.2.1.13.20")


aa_usefulCertificates = Attribute()
aa_usefulCertificates["attrType"] = id_kma_usefulCerts
aa_usefulCertificates["attrValues"][0] = CertificateSet()


# Key Wrap Attribute

id_kma_keyWrapAlgorithm = univ.ObjectIdentifier("2.16.840.1.101.2.1.13.21")


aa_keyWrapAlgorithm = Attribute()
aa_keyWrapAlgorithm["attrType"] = id_kma_keyWrapAlgorithm
aa_keyWrapAlgorithm["attrValues"][0] = AlgorithmIdentifier()


# Content Decryption Key Identifier Attribute

id_aa_KP_contentDecryptKeyID = univ.ObjectIdentifier("2.16.840.1.101.2.1.5.66")


class ContentDecryptKeyID(univ.OctetString):
    pass


aa_contentDecryptKeyIdentifier = Attribute()
aa_contentDecryptKeyIdentifier["attrType"] = id_aa_KP_contentDecryptKeyID
aa_contentDecryptKeyIdentifier["attrValues"][0] = ContentDecryptKeyID()


# Certificate Pointers Attribute

aa_certificatePointers = Attribute()
aa_certificatePointers["attrType"] = id_pe_subjectInfoAccess
aa_certificatePointers["attrValues"][0] = SubjectInfoAccessSyntax()


# CRL Pointers Attribute

id_aa_KP_crlPointers = univ.ObjectIdentifier("2.16.840.1.101.2.1.5.70")


aa_cRLDistributionPoints = Attribute()
aa_cRLDistributionPoints["attrType"] = id_aa_KP_crlPointers
aa_cRLDistributionPoints["attrValues"][0] = GeneralNames()


# Extended Error Codes

id_errorCodes = univ.ObjectIdentifier("2.16.840.1.101.2.1.22")

id_missingKeyType = univ.ObjectIdentifier("2.16.840.1.101.2.1.22.1")

id_privacyMarkTooLong = univ.ObjectIdentifier("2.16.840.1.101.2.1.22.2")

id_unrecognizedSecurityPolicy = univ.ObjectIdentifier("2.16.840.1.101.2.1.22.3")


# Map of Attribute Type OIDs to Attributes added to the
# ones that are in rfc5652.py

_cmsAttributesMapUpdate = {
    id_aa_contentHint: ContentHints(),
    id_aa_communityIdentifiers: CommunityIdentifiers(),
    id_aa_binarySigningTime: BinarySigningTime(),
    id_contentType: ContentType(),
    id_messageDigest: MessageDigest(),
    id_aa_KP_keyPkgIdAndReceiptReq: KeyPkgIdentifierAndReceiptReq(),
    id_aa_KP_keyProvinceV2: KeyProvinceV2(),
    id_aa_KP_manifest: Manifest(),
    id_kma_keyAlgorithm: KeyAlgorithm(),
    id_at_userCertificate: Certificate(),
    id_kma_keyPkgReceiversV2: KeyPkgReceiversV2(),
    id_kma_TSECNomenclature: TSECNomenclature(),
    id_kma_keyPurpose: KeyPurpose(),
    id_kma_keyUse: KeyUse(),
    id_kma_transportKey: TransOp(),
    id_kma_keyDistPeriod: KeyDistPeriod(),
    id_kma_keyValidityPeriod: KeyValidityPeriod(),
    id_kma_keyDuration: KeyDuration(),
    id_aa_KP_classification: Classification(),
    id_kma_splitID: SplitID(),
    id_kma_keyPkgType: KeyPkgType(),
    id_kma_sigUsageV3: SignatureUsage(),
    id_kma_otherCertFormats: CertificateChoices(),
    id_at_pkiPath: PkiPath(),
    id_kma_usefulCerts: CertificateSet(),
    id_kma_keyWrapAlgorithm: AlgorithmIdentifier(),
    id_aa_KP_contentDecryptKeyID: ContentDecryptKeyID(),
    id_pe_subjectInfoAccess: SubjectInfoAccessSyntax(),
    id_aa_KP_crlPointers: GeneralNames(),
}

rfc5652.cmsAttributesMap.update(_cmsAttributesMapUpdate)



#
# This file is part of pyasn1-modules software.
#
# Created by Russ Housley with assistance from asn1ate v.0.6.0.
#
# Copyright (c) 2019, Vigil Security, LLC
# License: https://www.pysnmp.com/pyasn1/license.html
#
# Time-Stamp Protocol (TSP)
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc3161.txt
#

from pyasn1.type import constraint, namedtype, namedval, tag, univ, useful

from pyasn1_modules import rfc4210, rfc5280, rfc5652

Extensions = rfc5280.Extensions

AlgorithmIdentifier = rfc5280.AlgorithmIdentifier

GeneralName = rfc5280.GeneralName

ContentInfo = rfc5652.ContentInfo

PKIFreeText = rfc4210.PKIFreeText


id_ct_TSTInfo = univ.ObjectIdentifier("1.2.840.113549.1.9.16.1.4")


class Accuracy(univ.Sequence):
    pass


Accuracy.componentType = namedtype.NamedTypes(
    namedtype.OptionalNamedType("seconds", univ.Integer()),
    namedtype.OptionalNamedType(
        "millis",
        univ.Integer()
        .subtype(subtypeSpec=constraint.ValueRangeConstraint(1, 999))
        .subtype(implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)),
    ),
    namedtype.OptionalNamedType(
        "micros",
        univ.Integer()
        .subtype(subtypeSpec=constraint.ValueRangeConstraint(1, 999))
        .subtype(implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)),
    ),
)


class MessageImprint(univ.Sequence):
    pass


MessageImprint.componentType = namedtype.NamedTypes(
    namedtype.NamedType("hashAlgorithm", AlgorithmIdentifier()),
    namedtype.NamedType("hashedMessage", univ.OctetString()),
)


class PKIFailureInfo(univ.BitString):
    pass


PKIFailureInfo.namedValues = namedval.NamedValues(
    ("badAlg", 0),
    ("badRequest", 2),
    ("badDataFormat", 5),
    ("timeNotAvailable", 14),
    ("unacceptedPolicy", 15),
    ("unacceptedExtension", 16),
    ("addInfoNotAvailable", 17),
    ("systemFailure", 25),
)


class PKIStatus(univ.Integer):
    pass


PKIStatus.namedValues = namedval.NamedValues(
    ("granted", 0),
    ("grantedWithMods", 1),
    ("rejection", 2),
    ("waiting", 3),
    ("revocationWarning", 4),
    ("revocationNotification", 5),
)


class PKIStatusInfo(univ.Sequence):
    pass


PKIStatusInfo.componentType = namedtype.NamedTypes(
    namedtype.NamedType("status", PKIStatus()),
    namedtype.OptionalNamedType("statusString", PKIFreeText()),
    namedtype.OptionalNamedType("failInfo", PKIFailureInfo()),
)


class TSAPolicyId(univ.ObjectIdentifier):
    pass


class TSTInfo(univ.Sequence):
    pass


TSTInfo.componentType = namedtype.NamedTypes(
    namedtype.NamedType(
        "version", univ.Integer(namedValues=namedval.NamedValues(("v1", 1)))
    ),
    namedtype.NamedType("policy", TSAPolicyId()),
    namedtype.NamedType("messageImprint", MessageImprint()),
    namedtype.NamedType("serialNumber", univ.Integer()),
    namedtype.NamedType("genTime", useful.GeneralizedTime()),
    namedtype.OptionalNamedType("accuracy", Accuracy()),
    namedtype.DefaultedNamedType("ordering", univ.Boolean().subtype(value=0)),
    namedtype.OptionalNamedType("nonce", univ.Integer()),
    namedtype.OptionalNamedType(
        "tsa",
        GeneralName().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
        ),
    ),
    namedtype.OptionalNamedType(
        "extensions",
        Extensions().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
        ),
    ),
)


class TimeStampReq(univ.Sequence):
    pass


TimeStampReq.componentType = namedtype.NamedTypes(
    namedtype.NamedType(
        "version", univ.Integer(namedValues=namedval.NamedValues(("v1", 1)))
    ),
    namedtype.NamedType("messageImprint", MessageImprint()),
    namedtype.OptionalNamedType("reqPolicy", TSAPolicyId()),
    namedtype.OptionalNamedType("nonce", univ.Integer()),
    namedtype.DefaultedNamedType("certReq", univ.Boolean().subtype(value=0)),
    namedtype.OptionalNamedType(
        "extensions",
        Extensions().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
        ),
    ),
)


class TimeStampToken(ContentInfo):
    pass


class TimeStampResp(univ.Sequence):
    pass


TimeStampResp.componentType = namedtype.NamedTypes(
    namedtype.NamedType("status", PKIStatusInfo()),
    namedtype.OptionalNamedType("timeStampToken", TimeStampToken()),
)



#
# This file is part of pyasn1-modules software.
#
# Created by Russ Housley with assistance from asn1ate v.0.6.0.
#
# Copyright (c) 2019, Vigil Security, LLC
# License: https://www.pysnmp.com/pyasn1/license.html
#
# Diffie-Hellman Key Agreement
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc3820.txt
#

from pyasn1.type import namedtype, univ

from pyasn1_modules import rfc5280


class ProxyCertPathLengthConstraint(univ.Integer):
    pass


class ProxyPolicy(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("policyLanguage", univ.ObjectIdentifier()),
        namedtype.OptionalNamedType("policy", univ.OctetString()),
    )


class ProxyCertInfoExtension(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.OptionalNamedType(
            "pCPathLenConstraint", ProxyCertPathLengthConstraint()
        ),
        namedtype.NamedType("proxyPolicy", ProxyPolicy()),
    )


id_pkix = univ.ObjectIdentifier(
    (
        1,
        3,
        6,
        1,
        5,
        5,
        7,
    )
)


id_pe = id_pkix + (1,)

id_pe_proxyCertInfo = id_pe + (14,)


id_ppl = id_pkix + (21,)

id_ppl_anyLanguage = id_ppl + (0,)

id_ppl_inheritAll = id_ppl + (1,)

id_ppl_independent = id_ppl + (2,)


# Map of Certificate Extension OIDs to Extensions added to the
# ones that are in rfc5280.py

_certificateExtensionsMapUpdate = {
    id_pe_proxyCertInfo: ProxyCertInfoExtension(),
}

rfc5280.certificateExtensionsMap.update(_certificateExtensionsMapUpdate)



#
# This file is part of pyasn1-modules software.
#
# Copyright (c) 2005-2020, Ilya Etingof <etingof@gmail.com>
# License: https://www.pysnmp.com/pyasn1/license.html
#
# SNMPv1 message syntax
#
# ASN.1 source from:
# http://www.ietf.org/rfc/rfc1155.txt
#
# Sample captures from:
# http://wiki.wireshark.org/SampleCaptures/
#
from pyasn1.type import constraint, namedtype, tag, univ


class ObjectName(univ.ObjectIdentifier):
    pass


class SimpleSyntax(univ.Choice):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("number", univ.Integer()),
        namedtype.NamedType("string", univ.OctetString()),
        namedtype.NamedType("object", univ.ObjectIdentifier()),
        namedtype.NamedType("empty", univ.Null()),
    )


class IpAddress(univ.OctetString):
    tagSet = univ.OctetString.tagSet.tagImplicitly(
        tag.Tag(tag.tagClassApplication, tag.tagFormatSimple, 0)
    )
    subtypeSpec = univ.Integer.subtypeSpec + constraint.ValueSizeConstraint(4, 4)


class NetworkAddress(univ.Choice):
    componentType = namedtype.NamedTypes(namedtype.NamedType("internet", IpAddress()))


class Counter(univ.Integer):
    tagSet = univ.Integer.tagSet.tagImplicitly(
        tag.Tag(tag.tagClassApplication, tag.tagFormatSimple, 1)
    )
    subtypeSpec = univ.Integer.subtypeSpec + constraint.ValueRangeConstraint(
        0, 4294967295
    )


class Gauge(univ.Integer):
    tagSet = univ.Integer.tagSet.tagImplicitly(
        tag.Tag(tag.tagClassApplication, tag.tagFormatSimple, 2)
    )
    subtypeSpec = univ.Integer.subtypeSpec + constraint.ValueRangeConstraint(
        0, 4294967295
    )


class TimeTicks(univ.Integer):
    tagSet = univ.Integer.tagSet.tagImplicitly(
        tag.Tag(tag.tagClassApplication, tag.tagFormatSimple, 3)
    )
    subtypeSpec = univ.Integer.subtypeSpec + constraint.ValueRangeConstraint(
        0, 4294967295
    )


class Opaque(univ.OctetString):
    tagSet = univ.OctetString.tagSet.tagImplicitly(
        tag.Tag(tag.tagClassApplication, tag.tagFormatSimple, 4)
    )


class ApplicationSyntax(univ.Choice):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("address", NetworkAddress()),
        namedtype.NamedType("counter", Counter()),
        namedtype.NamedType("gauge", Gauge()),
        namedtype.NamedType("ticks", TimeTicks()),
        namedtype.NamedType("arbitrary", Opaque()),
    )


class ObjectSyntax(univ.Choice):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("simple", SimpleSyntax()),
        namedtype.NamedType("application-wide", ApplicationSyntax()),
    )



# This file is being contributed to pyasn1-modules software.
#
# Created by Russ Housley with assistance from the asn1ate tool, with manual
#   changes to implement appropriate constraints and added comments.
# Modified by Russ Housley to add maps for use with opentypes.
#
# Copyright (c) 2019, Vigil Security, LLC
# License: https://www.pysnmp.com/pyasn1/license.html
#
# JWT Claim Constraints and TN Authorization List for certificate extensions.
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc8226.txt (with errata corrected)

from pyasn1.type import char, constraint, namedtype, tag, univ

from pyasn1_modules import rfc5280

MAX = float("inf")


def _OID(*components):
    output = []
    for x in tuple(components):
        if isinstance(x, univ.ObjectIdentifier):
            output.extend(list(x))
        else:
            output.append(int(x))

    return univ.ObjectIdentifier(output)


class JWTClaimName(char.IA5String):
    pass


class JWTClaimNames(univ.SequenceOf):
    pass


JWTClaimNames.componentType = JWTClaimName()
JWTClaimNames.sizeSpec = constraint.ValueSizeConstraint(1, MAX)


class JWTClaimPermittedValues(univ.Sequence):
    pass


JWTClaimPermittedValues.componentType = namedtype.NamedTypes(
    namedtype.NamedType("claim", JWTClaimName()),
    namedtype.NamedType(
        "permitted",
        univ.SequenceOf(componentType=char.UTF8String()).subtype(
            sizeSpec=constraint.ValueSizeConstraint(1, MAX)
        ),
    ),
)


class JWTClaimPermittedValuesList(univ.SequenceOf):
    pass


JWTClaimPermittedValuesList.componentType = JWTClaimPermittedValues()
JWTClaimPermittedValuesList.sizeSpec = constraint.ValueSizeConstraint(1, MAX)


class JWTClaimConstraints(univ.Sequence):
    pass


JWTClaimConstraints.componentType = namedtype.NamedTypes(
    namedtype.OptionalNamedType(
        "mustInclude",
        JWTClaimNames().subtype(
            explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
        ),
    ),
    namedtype.OptionalNamedType(
        "permittedValues",
        JWTClaimPermittedValuesList().subtype(
            explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
        ),
    ),
)

JWTClaimConstraints.subtypeSpec = constraint.ConstraintsUnion(
    constraint.WithComponentsConstraint(
        ("mustInclude", constraint.ComponentPresentConstraint())
    ),
    constraint.WithComponentsConstraint(
        ("permittedValues", constraint.ComponentPresentConstraint())
    ),
)


id_pe_JWTClaimConstraints = _OID(1, 3, 6, 1, 5, 5, 7, 1, 27)


class ServiceProviderCode(char.IA5String):
    pass


class TelephoneNumber(char.IA5String):
    pass


TelephoneNumber.subtypeSpec = constraint.ConstraintsIntersection(
    constraint.ValueSizeConstraint(1, 15),
    constraint.PermittedAlphabetConstraint(
        "0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "#", "*"
    ),
)


class TelephoneNumberRange(univ.Sequence):
    pass


TelephoneNumberRange.componentType = namedtype.NamedTypes(
    namedtype.NamedType("start", TelephoneNumber()),
    namedtype.NamedType(
        "count",
        univ.Integer().subtype(subtypeSpec=constraint.ValueRangeConstraint(2, MAX)),
    ),
)


class TNEntry(univ.Choice):
    pass


TNEntry.componentType = namedtype.NamedTypes(
    namedtype.NamedType(
        "spc",
        ServiceProviderCode().subtype(
            explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
        ),
    ),
    namedtype.NamedType(
        "range",
        TelephoneNumberRange().subtype(
            explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 1)
        ),
    ),
    namedtype.NamedType(
        "one",
        TelephoneNumber().subtype(
            explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 2)
        ),
    ),
)


class TNAuthorizationList(univ.SequenceOf):
    pass


TNAuthorizationList.componentType = TNEntry()
TNAuthorizationList.sizeSpec = constraint.ValueSizeConstraint(1, MAX)

id_pe_TNAuthList = _OID(1, 3, 6, 1, 5, 5, 7, 1, 26)


id_ad_stirTNList = _OID(1, 3, 6, 1, 5, 5, 7, 48, 14)


# Map of Certificate Extension OIDs to Extensions added to the
# ones that are in rfc5280.py

_certificateExtensionsMapUpdate = {
    id_pe_TNAuthList: TNAuthorizationList(),
    id_pe_JWTClaimConstraints: JWTClaimConstraints(),
}

rfc5280.certificateExtensionsMap.update(_certificateExtensionsMapUpdate)



#
# This file is part of pyasn1-modules software.
#
# Created by Russ Housley
# Copyright (c) 2019, Vigil Security, LLC
# License: https://www.pysnmp.com/pyasn1/license.html
#
import sys
import unittest

from pyasn1.codec.der.decoder import decode as der_decoder
from pyasn1.codec.der.encoder import encode as der_encoder

from pyasn1_modules import pem, rfc5280, rfc5990


class RSAKEMTestCase(unittest.TestCase):
    pem_text = """\
MEcGCyqGSIb3DQEJEAMOMDgwKQYHKIGMcQICBDAeMBkGCiuBBRCGSAksAQIwCwYJ
YIZIAWUDBAIBAgEQMAsGCWCGSAFlAwQBBQ==
"""

    def setUp(self):
        self.asn1Spec = rfc5280.AlgorithmIdentifier()

    def testDerCodec(self):
        substrate = pem.readBase64fromText(self.pem_text)
        asn1Object, rest = der_decoder(substrate, asn1Spec=self.asn1Spec)

        self.assertFalse(rest)
        self.assertTrue(asn1Object.prettyPrint())
        self.assertEqual(substrate, der_encoder(asn1Object))
        self.assertEqual(rfc5990.id_rsa_kem, asn1Object["algorithm"])

        rsa_kem_p, rest = der_decoder(
            asn1Object["parameters"],
            asn1Spec=rfc5280.algorithmIdentifierMap[rfc5990.id_rsa_kem],
        )

        self.assertFalse(rest)
        self.assertTrue(rsa_kem_p.prettyPrint())
        self.assertEqual(asn1Object["parameters"], der_encoder(rsa_kem_p))
        self.assertEqual(rfc5990.id_kem_rsa, rsa_kem_p["kem"]["algorithm"])

        kem_rsa_p, rest = der_decoder(
            rsa_kem_p["kem"]["parameters"],
            asn1Spec=rfc5280.algorithmIdentifierMap[rfc5990.id_kem_rsa],
        )

        self.assertFalse(rest)
        self.assertTrue(kem_rsa_p.prettyPrint())
        self.assertEqual(rsa_kem_p["kem"]["parameters"], der_encoder(kem_rsa_p))
        self.assertEqual(16, kem_rsa_p["keyLength"])
        self.assertEqual(
            rfc5990.id_kdf_kdf3, kem_rsa_p["keyDerivationFunction"]["algorithm"]
        )

        kdf_p, rest = der_decoder(
            kem_rsa_p["keyDerivationFunction"]["parameters"],
            asn1Spec=rfc5280.algorithmIdentifierMap[rfc5990.id_kdf_kdf3],
        )

        self.assertFalse(rest)
        self.assertTrue(kdf_p.prettyPrint())
        self.assertEqual(
            kem_rsa_p["keyDerivationFunction"]["parameters"], der_encoder(kdf_p)
        )

    def testOpenTypes(self):
        substrate = pem.readBase64fromText(self.pem_text)
        asn1Object, rest = der_decoder(
            substrate, asn1Spec=self.asn1Spec, decodeOpenTypes=True
        )

        self.assertFalse(rest)
        self.assertTrue(asn1Object.prettyPrint())
        self.assertEqual(substrate, der_encoder(asn1Object))
        self.assertEqual(rfc5990.id_rsa_kem, asn1Object["algorithm"])
        self.assertEqual(
            rfc5990.id_kem_rsa, asn1Object["parameters"]["kem"]["algorithm"]
        )
        self.assertEqual(16, asn1Object["parameters"]["kem"]["parameters"]["keyLength"])


suite = unittest.TestLoader().loadTestsFromModule(sys.modules[__name__])

if __name__ == "__main__":
    result = unittest.TextTestRunner(verbosity=2).run(suite)
    sys.exit(not result.wasSuccessful())



#
# This file is part of pyasn1-modules software.
#
# Created by Russ Housley with assistance from asn1ate v.0.6.0.
# Modified by Russ Housley to add a map for use with opentypes.
#
# Copyright (c) 2019, Vigil Security, LLC
# License: https://www.pysnmp.com/pyasn1/license.html
#
# Enhanced Security Services for S/MIME
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc2634.txt
#

from pyasn1.type import char, constraint, namedtype, namedval, tag, univ, useful

from pyasn1_modules import rfc5280, rfc5652

MAX = float("inf")

ContentType = rfc5652.ContentType

IssuerAndSerialNumber = rfc5652.IssuerAndSerialNumber

SubjectKeyIdentifier = rfc5652.SubjectKeyIdentifier

PolicyInformation = rfc5280.PolicyInformation

GeneralNames = rfc5280.GeneralNames

CertificateSerialNumber = rfc5280.CertificateSerialNumber


# Signing Certificate Attribute
# Warning: It is better to use SigningCertificateV2 from RFC 5035

id_aa_signingCertificate = univ.ObjectIdentifier("1.2.840.113549.1.9.16.2.12")


class Hash(univ.OctetString):
    pass  # SHA-1 hash of entire certificate; RFC 5035 supports other hash algorithms


class IssuerSerial(univ.Sequence):
    pass


IssuerSerial.componentType = namedtype.NamedTypes(
    namedtype.NamedType("issuer", GeneralNames()),
    namedtype.NamedType("serialNumber", CertificateSerialNumber()),
)


class ESSCertID(univ.Sequence):
    pass


ESSCertID.componentType = namedtype.NamedTypes(
    namedtype.NamedType("certHash", Hash()),
    namedtype.OptionalNamedType("issuerSerial", IssuerSerial()),
)


class SigningCertificate(univ.Sequence):
    pass


SigningCertificate.componentType = namedtype.NamedTypes(
    namedtype.NamedType("certs", univ.SequenceOf(componentType=ESSCertID())),
    namedtype.OptionalNamedType(
        "policies", univ.SequenceOf(componentType=PolicyInformation())
    ),
)


# Mail List Expansion History Attribute

id_aa_mlExpandHistory = univ.ObjectIdentifier("1.2.840.113549.1.9.16.2.3")

ub_ml_expansion_history = univ.Integer(64)


class EntityIdentifier(univ.Choice):
    pass


EntityIdentifier.componentType = namedtype.NamedTypes(
    namedtype.NamedType("issuerAndSerialNumber", IssuerAndSerialNumber()),
    namedtype.NamedType("subjectKeyIdentifier", SubjectKeyIdentifier()),
)


class MLReceiptPolicy(univ.Choice):
    pass


MLReceiptPolicy.componentType = namedtype.NamedTypes(
    namedtype.NamedType(
        "none",
        univ.Null().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
        ),
    ),
    namedtype.NamedType(
        "insteadOf",
        univ.SequenceOf(componentType=GeneralNames())
        .subtype(sizeSpec=constraint.ValueSizeConstraint(1, MAX))
        .subtype(implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)),
    ),
    namedtype.NamedType(
        "inAdditionTo",
        univ.SequenceOf(componentType=GeneralNames())
        .subtype(sizeSpec=constraint.ValueSizeConstraint(1, MAX))
        .subtype(implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 2)),
    ),
)


class MLData(univ.Sequence):
    pass


MLData.componentType = namedtype.NamedTypes(
    namedtype.NamedType("mailListIdentifier", EntityIdentifier()),
    namedtype.NamedType("expansionTime", useful.GeneralizedTime()),
    namedtype.OptionalNamedType("mlReceiptPolicy", MLReceiptPolicy()),
)


class MLExpansionHistory(univ.SequenceOf):
    pass


MLExpansionHistory.componentType = MLData()
MLExpansionHistory.sizeSpec = constraint.ValueSizeConstraint(1, ub_ml_expansion_history)


# ESS Security Label Attribute

id_aa_securityLabel = univ.ObjectIdentifier("1.2.840.113549.1.9.16.2.2")

ub_privacy_mark_length = univ.Integer(128)

ub_security_categories = univ.Integer(64)

ub_integer_options = univ.Integer(256)


class ESSPrivacyMark(univ.Choice):
    pass


ESSPrivacyMark.componentType = namedtype.NamedTypes(
    namedtype.NamedType(
        "pString",
        char.PrintableString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_privacy_mark_length)
        ),
    ),
    namedtype.NamedType(
        "utf8String",
        char.UTF8String().subtype(subtypeSpec=constraint.ValueSizeConstraint(1, MAX)),
    ),
)


class SecurityClassification(univ.Integer):
    pass


SecurityClassification.subtypeSpec = constraint.ValueRangeConstraint(
    0, ub_integer_options
)

SecurityClassification.namedValues = namedval.NamedValues(
    ("unmarked", 0),
    ("unclassified", 1),
    ("restricted", 2),
    ("confidential", 3),
    ("secret", 4),
    ("top-secret", 5),
)


class SecurityPolicyIdentifier(univ.ObjectIdentifier):
    pass


class SecurityCategory(univ.Sequence):
    pass


SecurityCategory.componentType = namedtype.NamedTypes(
    namedtype.NamedType(
        "type",
        univ.ObjectIdentifier().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
        ),
    ),
    namedtype.NamedType(
        "value",
        univ.Any().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
        ),
    ),
)


class SecurityCategories(univ.SetOf):
    pass


SecurityCategories.componentType = SecurityCategory()
SecurityCategories.sizeSpec = constraint.ValueSizeConstraint(1, ub_security_categories)


class ESSSecurityLabel(univ.Set):
    pass


ESSSecurityLabel.componentType = namedtype.NamedTypes(
    namedtype.NamedType("security-policy-identifier", SecurityPolicyIdentifier()),
    namedtype.OptionalNamedType("security-classification", SecurityClassification()),
    namedtype.OptionalNamedType("privacy-mark", ESSPrivacyMark()),
    namedtype.OptionalNamedType("security-categories", SecurityCategories()),
)


# Equivalent Labels Attribute

id_aa_equivalentLabels = univ.ObjectIdentifier("1.2.840.113549.1.9.16.2.9")


class EquivalentLabels(univ.SequenceOf):
    pass


EquivalentLabels.componentType = ESSSecurityLabel()


# Content Identifier Attribute

id_aa_contentIdentifier = univ.ObjectIdentifier("1.2.840.113549.1.9.16.2.7")


class ContentIdentifier(univ.OctetString):
    pass


# Content Reference Attribute

id_aa_contentReference = univ.ObjectIdentifier("1.2.840.113549.1.9.16.2.10")


class ContentReference(univ.Sequence):
    pass


ContentReference.componentType = namedtype.NamedTypes(
    namedtype.NamedType("contentType", ContentType()),
    namedtype.NamedType("signedContentIdentifier", ContentIdentifier()),
    namedtype.NamedType("originatorSignatureValue", univ.OctetString()),
)


# Message Signature Digest Attribute

id_aa_msgSigDigest = univ.ObjectIdentifier("1.2.840.113549.1.9.16.2.5")


class MsgSigDigest(univ.OctetString):
    pass


# Content Hints Attribute

id_aa_contentHint = univ.ObjectIdentifier("1.2.840.113549.1.9.16.2.4")


class ContentHints(univ.Sequence):
    pass


ContentHints.componentType = namedtype.NamedTypes(
    namedtype.OptionalNamedType(
        "contentDescription",
        char.UTF8String().subtype(subtypeSpec=constraint.ValueSizeConstraint(1, MAX)),
    ),
    namedtype.NamedType("contentType", ContentType()),
)


# Receipt Request Attribute


class AllOrFirstTier(univ.Integer):
    pass


AllOrFirstTier.namedValues = namedval.NamedValues(
    ("allReceipts", 0), ("firstTierRecipients", 1)
)


class ReceiptsFrom(univ.Choice):
    pass


ReceiptsFrom.componentType = namedtype.NamedTypes(
    namedtype.NamedType(
        "allOrFirstTier",
        AllOrFirstTier().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
        ),
    ),
    namedtype.NamedType(
        "receiptList",
        univ.SequenceOf(componentType=GeneralNames()).subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
        ),
    ),
)


id_aa_receiptRequest = univ.ObjectIdentifier("1.2.840.113549.1.9.16.2.1")

ub_receiptsTo = univ.Integer(16)


class ReceiptRequest(univ.Sequence):
    pass


ReceiptRequest.componentType = namedtype.NamedTypes(
    namedtype.NamedType("signedContentIdentifier", ContentIdentifier()),
    namedtype.NamedType("receiptsFrom", ReceiptsFrom()),
    namedtype.NamedType(
        "receiptsTo",
        univ.SequenceOf(componentType=GeneralNames()).subtype(
            sizeSpec=constraint.ValueSizeConstraint(1, ub_receiptsTo)
        ),
    ),
)

# Receipt Content Type


class ESSVersion(univ.Integer):
    pass


ESSVersion.namedValues = namedval.NamedValues(("v1", 1))


id_ct_receipt = univ.ObjectIdentifier("1.2.840.113549.1.9.16.1.1")


class Receipt(univ.Sequence):
    pass


Receipt.componentType = namedtype.NamedTypes(
    namedtype.NamedType("version", ESSVersion()),
    namedtype.NamedType("contentType", ContentType()),
    namedtype.NamedType("signedContentIdentifier", ContentIdentifier()),
    namedtype.NamedType("originatorSignatureValue", univ.OctetString()),
)


# Map of Attribute Type to the Attribute structure is added to the
# ones that are in rfc5652.py

_cmsAttributesMapUpdate = {
    id_aa_signingCertificate: SigningCertificate(),
    id_aa_mlExpandHistory: MLExpansionHistory(),
    id_aa_securityLabel: ESSSecurityLabel(),
    id_aa_equivalentLabels: EquivalentLabels(),
    id_aa_contentIdentifier: ContentIdentifier(),
    id_aa_contentReference: ContentReference(),
    id_aa_msgSigDigest: MsgSigDigest(),
    id_aa_contentHint: ContentHints(),
    id_aa_receiptRequest: ReceiptRequest(),
}

rfc5652.cmsAttributesMap.update(_cmsAttributesMapUpdate)


# Map of Content Type OIDs to Content Types is added to the
# ones that are in rfc5652.py

_cmsContentTypesMapUpdate = {
    id_ct_receipt: Receipt(),
}

rfc5652.cmsContentTypesMap.update(_cmsContentTypesMapUpdate)



#
# This file is part of pyasn1-modules software.
#
# Created by Russ Housley.
#
# Copyright (c) 2019, Vigil Security, LLC
# License: https://www.pysnmp.com/pyasn1/license.html
#
# Online Certificate Status Protocol (OCSP)
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc6960.txt
#

from pyasn1.type import char, constraint, namedtype, namedval, tag, univ, useful

from pyasn1_modules import rfc2560, rfc5280

MAX = float("inf")


# Imports from RFC 5280

AlgorithmIdentifier = rfc5280.AlgorithmIdentifier
AuthorityInfoAccessSyntax = rfc5280.AuthorityInfoAccessSyntax
Certificate = rfc5280.Certificate
CertificateSerialNumber = rfc5280.CertificateSerialNumber
CRLReason = rfc5280.CRLReason
Extensions = rfc5280.Extensions
GeneralName = rfc5280.GeneralName
Name = rfc5280.Name

id_kp = rfc5280.id_kp

id_ad_ocsp = rfc5280.id_ad_ocsp


# Imports from the original OCSP module in RFC 2560

AcceptableResponses = rfc2560.AcceptableResponses
ArchiveCutoff = rfc2560.ArchiveCutoff
CertStatus = rfc2560.CertStatus
KeyHash = rfc2560.KeyHash
OCSPResponse = rfc2560.OCSPResponse
OCSPResponseStatus = rfc2560.OCSPResponseStatus
ResponseBytes = rfc2560.ResponseBytes
RevokedInfo = rfc2560.RevokedInfo
UnknownInfo = rfc2560.UnknownInfo
Version = rfc2560.Version

id_kp_OCSPSigning = rfc2560.id_kp_OCSPSigning

id_pkix_ocsp = rfc2560.id_pkix_ocsp
id_pkix_ocsp_archive_cutoff = rfc2560.id_pkix_ocsp_archive_cutoff
id_pkix_ocsp_basic = rfc2560.id_pkix_ocsp_basic
id_pkix_ocsp_crl = rfc2560.id_pkix_ocsp_crl
id_pkix_ocsp_nocheck = rfc2560.id_pkix_ocsp_nocheck
id_pkix_ocsp_nonce = rfc2560.id_pkix_ocsp_nonce
id_pkix_ocsp_response = rfc2560.id_pkix_ocsp_response
id_pkix_ocsp_service_locator = rfc2560.id_pkix_ocsp_service_locator


# Additional object identifiers

id_pkix_ocsp_pref_sig_algs = id_pkix_ocsp + (8,)
id_pkix_ocsp_extended_revoke = id_pkix_ocsp + (9,)


# Updated structures (mostly to improve openTypes support)


class CertID(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("hashAlgorithm", AlgorithmIdentifier()),
        namedtype.NamedType("issuerNameHash", univ.OctetString()),
        namedtype.NamedType("issuerKeyHash", univ.OctetString()),
        namedtype.NamedType("serialNumber", CertificateSerialNumber()),
    )


class SingleResponse(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("certID", CertID()),
        namedtype.NamedType("certStatus", CertStatus()),
        namedtype.NamedType("thisUpdate", useful.GeneralizedTime()),
        namedtype.OptionalNamedType(
            "nextUpdate",
            useful.GeneralizedTime().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
            ),
        ),
        namedtype.OptionalNamedType(
            "singleExtensions",
            Extensions().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
            ),
        ),
    )


class ResponderID(univ.Choice):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType(
            "byName",
            Name().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
            ),
        ),
        namedtype.NamedType(
            "byKey",
            KeyHash().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 2)
            ),
        ),
    )


class ResponseData(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.DefaultedNamedType(
            "version",
            Version("v1").subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
            ),
        ),
        namedtype.NamedType("responderID", ResponderID()),
        namedtype.NamedType("producedAt", useful.GeneralizedTime()),
        namedtype.NamedType(
            "responses", univ.SequenceOf(componentType=SingleResponse())
        ),
        namedtype.OptionalNamedType(
            "responseExtensions",
            Extensions().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
            ),
        ),
    )


class BasicOCSPResponse(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("tbsResponseData", ResponseData()),
        namedtype.NamedType("signatureAlgorithm", AlgorithmIdentifier()),
        namedtype.NamedType("signature", univ.BitString()),
        namedtype.OptionalNamedType(
            "certs",
            univ.SequenceOf(componentType=Certificate()).subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
            ),
        ),
    )


class Request(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("reqCert", CertID()),
        namedtype.OptionalNamedType(
            "singleRequestExtensions",
            Extensions().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
            ),
        ),
    )


class Signature(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("signatureAlgorithm", AlgorithmIdentifier()),
        namedtype.NamedType("signature", univ.BitString()),
        namedtype.OptionalNamedType(
            "certs",
            univ.SequenceOf(componentType=Certificate()).subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
            ),
        ),
    )


class TBSRequest(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.DefaultedNamedType(
            "version",
            Version("v1").subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
            ),
        ),
        namedtype.OptionalNamedType(
            "requestorName",
            GeneralName().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
            ),
        ),
        namedtype.NamedType("requestList", univ.SequenceOf(componentType=Request())),
        namedtype.OptionalNamedType(
            "requestExtensions",
            Extensions().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 2)
            ),
        ),
    )


class OCSPRequest(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("tbsRequest", TBSRequest()),
        namedtype.OptionalNamedType(
            "optionalSignature",
            Signature().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
            ),
        ),
    )


# Previously omitted structure


class ServiceLocator(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("issuer", Name()),
        namedtype.NamedType("locator", AuthorityInfoAccessSyntax()),
    )


# Additional structures


class CrlID(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.OptionalNamedType(
            "crlUrl",
            char.IA5String().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
            ),
        ),
        namedtype.OptionalNamedType(
            "crlNum",
            univ.Integer().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
            ),
        ),
        namedtype.OptionalNamedType(
            "crlTime",
            useful.GeneralizedTime().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 2)
            ),
        ),
    )


class PreferredSignatureAlgorithm(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("sigIdentifier", AlgorithmIdentifier()),
        namedtype.OptionalNamedType("certIdentifier", AlgorithmIdentifier()),
    )


class PreferredSignatureAlgorithms(univ.SequenceOf):
    componentType = PreferredSignatureAlgorithm()


# Response Type OID to Response Map

ocspResponseMap = {
    id_pkix_ocsp_basic: BasicOCSPResponse(),
}


# Map of Extension OIDs to Extensions added to the ones
# that are in rfc5280.py

_certificateExtensionsMapUpdate = {
    # Certificate Extension
    id_pkix_ocsp_nocheck: univ.Null(""),
    # OCSP Request Extensions
    id_pkix_ocsp_nonce: univ.OctetString(),
    id_pkix_ocsp_response: AcceptableResponses(),
    id_pkix_ocsp_service_locator: ServiceLocator(),
    id_pkix_ocsp_pref_sig_algs: PreferredSignatureAlgorithms(),
    # OCSP Response Extensions
    id_pkix_ocsp_crl: CrlID(),
    id_pkix_ocsp_archive_cutoff: ArchiveCutoff(),
    id_pkix_ocsp_extended_revoke: univ.Null(""),
}

rfc5280.certificateExtensionsMap.update(_certificateExtensionsMapUpdate)



#
# This file is part of pyasn1-modules software.
#
# Copyright (c) 2005-2020, Ilya Etingof <etingof@gmail.com>
# License: https://www.pysnmp.com/pyasn1/license.html
#
# LDAP message syntax
#
# ASN.1 source from:
# http://www.trl.ibm.com/projects/xml/xss4j/data/asn1/grammars/ldap.asn
#
# Sample captures from:
# http://wiki.wireshark.org/SampleCaptures/
#
from pyasn1.type import constraint, namedtype, namedval, tag, univ

maxInt = univ.Integer(2147483647)


class LDAPString(univ.OctetString):
    pass


class LDAPOID(univ.OctetString):
    pass


class LDAPDN(LDAPString):
    pass


class RelativeLDAPDN(LDAPString):
    pass


class AttributeType(LDAPString):
    pass


class AttributeDescription(LDAPString):
    pass


class AttributeDescriptionList(univ.SequenceOf):
    componentType = AttributeDescription()


class AttributeValue(univ.OctetString):
    pass


class AssertionValue(univ.OctetString):
    pass


class AttributeValueAssertion(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("attributeDesc", AttributeDescription()),
        namedtype.NamedType("assertionValue", AssertionValue()),
    )


class Attribute(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("type", AttributeDescription()),
        namedtype.NamedType("vals", univ.SetOf(componentType=AttributeValue())),
    )


class MatchingRuleId(LDAPString):
    pass


class Control(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("controlType", LDAPOID()),
        namedtype.DefaultedNamedType("criticality", univ.Boolean("False")),
        namedtype.OptionalNamedType("controlValue", univ.OctetString()),
    )


class Controls(univ.SequenceOf):
    componentType = Control()


class LDAPURL(LDAPString):
    pass


class Referral(univ.SequenceOf):
    componentType = LDAPURL()


class SaslCredentials(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("mechanism", LDAPString()),
        namedtype.OptionalNamedType("credentials", univ.OctetString()),
    )


class AuthenticationChoice(univ.Choice):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType(
            "simple",
            univ.OctetString().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
            ),
        ),
        namedtype.NamedType(
            "reserved-1",
            univ.OctetString().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
            ),
        ),
        namedtype.NamedType(
            "reserved-2",
            univ.OctetString().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 2)
            ),
        ),
        namedtype.NamedType(
            "sasl",
            SaslCredentials().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 3)
            ),
        ),
    )


class BindRequest(univ.Sequence):
    tagSet = univ.Sequence.tagSet.tagImplicitly(
        tag.Tag(tag.tagClassApplication, tag.tagFormatConstructed, 0)
    )
    componentType = namedtype.NamedTypes(
        namedtype.NamedType(
            "version",
            univ.Integer().subtype(subtypeSpec=constraint.ValueRangeConstraint(1, 127)),
        ),
        namedtype.NamedType("name", LDAPDN()),
        namedtype.NamedType("authentication", AuthenticationChoice()),
    )


class PartialAttributeList(univ.SequenceOf):
    componentType = univ.Sequence(
        componentType=namedtype.NamedTypes(
            namedtype.NamedType("type", AttributeDescription()),
            namedtype.NamedType("vals", univ.SetOf(componentType=AttributeValue())),
        )
    )


class SearchResultEntry(univ.Sequence):
    tagSet = univ.Sequence.tagSet.tagImplicitly(
        tag.Tag(tag.tagClassApplication, tag.tagFormatConstructed, 4)
    )
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("objectName", LDAPDN()),
        namedtype.NamedType("attributes", PartialAttributeList()),
    )


class MatchingRuleAssertion(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.OptionalNamedType(
            "matchingRule",
            MatchingRuleId().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
            ),
        ),
        namedtype.OptionalNamedType(
            "type",
            AttributeDescription().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 2)
            ),
        ),
        namedtype.NamedType(
            "matchValue",
            AssertionValue().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 3)
            ),
        ),
        namedtype.DefaultedNamedType(
            "dnAttributes",
            univ.Boolean("False").subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 4)
            ),
        ),
    )


class SubstringFilter(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("type", AttributeDescription()),
        namedtype.NamedType(
            "substrings",
            univ.SequenceOf(
                componentType=univ.Choice(
                    componentType=namedtype.NamedTypes(
                        namedtype.NamedType(
                            "initial",
                            LDAPString().subtype(
                                implicitTag=tag.Tag(
                                    tag.tagClassContext, tag.tagFormatSimple, 0
                                )
                            ),
                        ),
                        namedtype.NamedType(
                            "any",
                            LDAPString().subtype(
                                implicitTag=tag.Tag(
                                    tag.tagClassContext, tag.tagFormatSimple, 1
                                )
                            ),
                        ),
                        namedtype.NamedType(
                            "final",
                            LDAPString().subtype(
                                implicitTag=tag.Tag(
                                    tag.tagClassContext, tag.tagFormatSimple, 2
                                )
                            ),
                        ),
                    )
                )
            ),
        ),
    )


# Ugly hack to handle recursive Filter reference (up to 3-levels deep).


class Filter3(univ.Choice):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType(
            "equalityMatch",
            AttributeValueAssertion().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 3)
            ),
        ),
        namedtype.NamedType(
            "substrings",
            SubstringFilter().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 4)
            ),
        ),
        namedtype.NamedType(
            "greaterOrEqual",
            AttributeValueAssertion().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 5)
            ),
        ),
        namedtype.NamedType(
            "lessOrEqual",
            AttributeValueAssertion().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 6)
            ),
        ),
        namedtype.NamedType(
            "present",
            AttributeDescription().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 7)
            ),
        ),
        namedtype.NamedType(
            "approxMatch",
            AttributeValueAssertion().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 8)
            ),
        ),
        namedtype.NamedType(
            "extensibleMatch",
            MatchingRuleAssertion().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 9)
            ),
        ),
    )


class Filter2(univ.Choice):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType(
            "and",
            univ.SetOf(componentType=Filter3()).subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 0)
            ),
        ),
        namedtype.NamedType(
            "or",
            univ.SetOf(componentType=Filter3()).subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 1)
            ),
        ),
        namedtype.NamedType(
            "not",
            Filter3().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 2)
            ),
        ),
        namedtype.NamedType(
            "equalityMatch",
            AttributeValueAssertion().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 3)
            ),
        ),
        namedtype.NamedType(
            "substrings",
            SubstringFilter().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 4)
            ),
        ),
        namedtype.NamedType(
            "greaterOrEqual",
            AttributeValueAssertion().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 5)
            ),
        ),
        namedtype.NamedType(
            "lessOrEqual",
            AttributeValueAssertion().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 6)
            ),
        ),
        namedtype.NamedType(
            "present",
            AttributeDescription().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 7)
            ),
        ),
        namedtype.NamedType(
            "approxMatch",
            AttributeValueAssertion().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 8)
            ),
        ),
        namedtype.NamedType(
            "extensibleMatch",
            MatchingRuleAssertion().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 9)
            ),
        ),
    )


class Filter(univ.Choice):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType(
            "and",
            univ.SetOf(componentType=Filter2()).subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 0)
            ),
        ),
        namedtype.NamedType(
            "or",
            univ.SetOf(componentType=Filter2()).subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 1)
            ),
        ),
        namedtype.NamedType(
            "not",
            Filter2().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 2)
            ),
        ),
        namedtype.NamedType(
            "equalityMatch",
            AttributeValueAssertion().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 3)
            ),
        ),
        namedtype.NamedType(
            "substrings",
            SubstringFilter().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 4)
            ),
        ),
        namedtype.NamedType(
            "greaterOrEqual",
            AttributeValueAssertion().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 5)
            ),
        ),
        namedtype.NamedType(
            "lessOrEqual",
            AttributeValueAssertion().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 6)
            ),
        ),
        namedtype.NamedType(
            "present",
            AttributeDescription().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 7)
            ),
        ),
        namedtype.NamedType(
            "approxMatch",
            AttributeValueAssertion().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 8)
            ),
        ),
        namedtype.NamedType(
            "extensibleMatch",
            MatchingRuleAssertion().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 9)
            ),
        ),
    )


# End of Filter hack


class SearchRequest(univ.Sequence):
    tagSet = univ.Sequence.tagSet.tagImplicitly(
        tag.Tag(tag.tagClassApplication, tag.tagFormatConstructed, 3)
    )
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("baseObject", LDAPDN()),
        namedtype.NamedType(
            "scope",
            univ.Enumerated(
                namedValues=namedval.NamedValues(
                    ("baseObject", 0), ("singleLevel", 1), ("wholeSubtree", 2)
                )
            ),
        ),
        namedtype.NamedType(
            "derefAliases",
            univ.Enumerated(
                namedValues=namedval.NamedValues(
                    ("neverDerefAliases", 0),
                    ("derefInSearching", 1),
                    ("derefFindingBaseObj", 2),
                    ("derefAlways", 3),
                )
            ),
        ),
        namedtype.NamedType(
            "sizeLimit",
            univ.Integer().subtype(
                subtypeSpec=constraint.ValueRangeConstraint(0, maxInt)
            ),
        ),
        namedtype.NamedType(
            "timeLimit",
            univ.Integer().subtype(
                subtypeSpec=constraint.ValueRangeConstraint(0, maxInt)
            ),
        ),
        namedtype.NamedType("typesOnly", univ.Boolean()),
        namedtype.NamedType("filter", Filter()),
        namedtype.NamedType("attributes", AttributeDescriptionList()),
    )


class UnbindRequest(univ.Null):
    tagSet = univ.Sequence.tagSet.tagImplicitly(
        tag.Tag(tag.tagClassApplication, tag.tagFormatSimple, 2)
    )


class BindResponse(univ.Sequence):
    tagSet = univ.Sequence.tagSet.tagImplicitly(
        tag.Tag(tag.tagClassApplication, tag.tagFormatConstructed, 1)
    )
    componentType = namedtype.NamedTypes(
        namedtype.NamedType(
            "resultCode",
            univ.Enumerated(
                namedValues=namedval.NamedValues(
                    ("success", 0),
                    ("operationsError", 1),
                    ("protocolError", 2),
                    ("timeLimitExceeded", 3),
                    ("sizeLimitExceeded", 4),
                    ("compareFalse", 5),
                    ("compareTrue", 6),
                    ("authMethodNotSupported", 7),
                    ("strongAuthRequired", 8),
                    ("reserved-9", 9),
                    ("referral", 10),
                    ("adminLimitExceeded", 11),
                    ("unavailableCriticalExtension", 12),
                    ("confidentialityRequired", 13),
                    ("saslBindInProgress", 14),
                    ("noSuchAttribute", 16),
                    ("undefinedAttributeType", 17),
                    ("inappropriateMatching", 18),
                    ("constraintViolation", 19),
                    ("attributeOrValueExists", 20),
                    ("invalidAttributeSyntax", 21),
                    ("noSuchObject", 32),
                    ("aliasProblem", 33),
                    ("invalidDNSyntax", 34),
                    ("reserved-35", 35),
                    ("aliasDereferencingProblem", 36),
                    ("inappropriateAuthentication", 48),
                    ("invalidCredentials", 49),
                    ("insufficientAccessRights", 50),
                    ("busy", 51),
                    ("unavailable", 52),
                    ("unwillingToPerform", 53),
                    ("loopDetect", 54),
                    ("namingViolation", 64),
                    ("objectClassViolation", 65),
                    ("notAllowedOnNonLeaf", 66),
                    ("notAllowedOnRDN", 67),
                    ("entryAlreadyExists", 68),
                    ("objectClassModsProhibited", 69),
                    ("reserved-70", 70),
                    ("affectsMultipleDSAs", 71),
                    ("other", 80),
                    ("reserved-81", 81),
                    ("reserved-82", 82),
                    ("reserved-83", 83),
                    ("reserved-84", 84),
                    ("reserved-85", 85),
                    ("reserved-86", 86),
                    ("reserved-87", 87),
                    ("reserved-88", 88),
                    ("reserved-89", 89),
                    ("reserved-90", 90),
                )
            ),
        ),
        namedtype.NamedType("matchedDN", LDAPDN()),
        namedtype.NamedType("errorMessage", LDAPString()),
        namedtype.OptionalNamedType(
            "referral",
            Referral().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 3)
            ),
        ),
        namedtype.OptionalNamedType(
            "serverSaslCreds",
            univ.OctetString().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 7)
            ),
        ),
    )


class LDAPResult(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType(
            "resultCode",
            univ.Enumerated(
                namedValues=namedval.NamedValues(
                    ("success", 0),
                    ("operationsError", 1),
                    ("protocolError", 2),
                    ("timeLimitExceeded", 3),
                    ("sizeLimitExceeded", 4),
                    ("compareFalse", 5),
                    ("compareTrue", 6),
                    ("authMethodNotSupported", 7),
                    ("strongAuthRequired", 8),
                    ("reserved-9", 9),
                    ("referral", 10),
                    ("adminLimitExceeded", 11),
                    ("unavailableCriticalExtension", 12),
                    ("confidentialityRequired", 13),
                    ("saslBindInProgress", 14),
                    ("noSuchAttribute", 16),
                    ("undefinedAttributeType", 17),
                    ("inappropriateMatching", 18),
                    ("constraintViolation", 19),
                    ("attributeOrValueExists", 20),
                    ("invalidAttributeSyntax", 21),
                    ("noSuchObject", 32),
                    ("aliasProblem", 33),
                    ("invalidDNSyntax", 34),
                    ("reserved-35", 35),
                    ("aliasDereferencingProblem", 36),
                    ("inappropriateAuthentication", 48),
                    ("invalidCredentials", 49),
                    ("insufficientAccessRights", 50),
                    ("busy", 51),
                    ("unavailable", 52),
                    ("unwillingToPerform", 53),
                    ("loopDetect", 54),
                    ("namingViolation", 64),
                    ("objectClassViolation", 65),
                    ("notAllowedOnNonLeaf", 66),
                    ("notAllowedOnRDN", 67),
                    ("entryAlreadyExists", 68),
                    ("objectClassModsProhibited", 69),
                    ("reserved-70", 70),
                    ("affectsMultipleDSAs", 71),
                    ("other", 80),
                    ("reserved-81", 81),
                    ("reserved-82", 82),
                    ("reserved-83", 83),
                    ("reserved-84", 84),
                    ("reserved-85", 85),
                    ("reserved-86", 86),
                    ("reserved-87", 87),
                    ("reserved-88", 88),
                    ("reserved-89", 89),
                    ("reserved-90", 90),
                )
            ),
        ),
        namedtype.NamedType("matchedDN", LDAPDN()),
        namedtype.NamedType("errorMessage", LDAPString()),
        namedtype.OptionalNamedType(
            "referral",
            Referral().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 3)
            ),
        ),
    )


class SearchResultReference(univ.SequenceOf):
    tagSet = univ.Sequence.tagSet.tagImplicitly(
        tag.Tag(tag.tagClassApplication, tag.tagFormatConstructed, 19)
    )
    componentType = LDAPURL()


class SearchResultDone(LDAPResult):
    tagSet = univ.Sequence.tagSet.tagImplicitly(
        tag.Tag(tag.tagClassApplication, tag.tagFormatConstructed, 5)
    )


class AttributeTypeAndValues(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("type", AttributeDescription()),
        namedtype.NamedType("vals", univ.SetOf(componentType=AttributeValue())),
    )


class ModifyRequest(univ.Sequence):
    tagSet = univ.Sequence.tagSet.tagImplicitly(
        tag.Tag(tag.tagClassApplication, tag.tagFormatConstructed, 6)
    )
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("object", LDAPDN()),
        namedtype.NamedType(
            "modification",
            univ.SequenceOf(
                componentType=univ.Sequence(
                    componentType=namedtype.NamedTypes(
                        namedtype.NamedType(
                            "operation",
                            univ.Enumerated(
                                namedValues=namedval.NamedValues(
                                    ("add", 0), ("delete", 1), ("replace", 2)
                                )
                            ),
                        ),
                        namedtype.NamedType("modification", AttributeTypeAndValues()),
                    )
                )
            ),
        ),
    )


class ModifyResponse(LDAPResult):
    tagSet = univ.Sequence.tagSet.tagImplicitly(
        tag.Tag(tag.tagClassApplication, tag.tagFormatConstructed, 7)
    )


class AttributeList(univ.SequenceOf):
    componentType = univ.Sequence(
        componentType=namedtype.NamedTypes(
            namedtype.NamedType("type", AttributeDescription()),
            namedtype.NamedType("vals", univ.SetOf(componentType=AttributeValue())),
        )
    )


class AddRequest(univ.Sequence):
    tagSet = univ.Sequence.tagSet.tagImplicitly(
        tag.Tag(tag.tagClassApplication, tag.tagFormatConstructed, 8)
    )
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("entry", LDAPDN()),
        namedtype.NamedType("attributes", AttributeList()),
    )


class AddResponse(LDAPResult):
    tagSet = univ.Sequence.tagSet.tagImplicitly(
        tag.Tag(tag.tagClassApplication, tag.tagFormatConstructed, 9)
    )


class DelRequest(LDAPResult):
    tagSet = univ.Sequence.tagSet.tagImplicitly(
        tag.Tag(tag.tagClassApplication, tag.tagFormatConstructed, 10)
    )


class DelResponse(LDAPResult):
    tagSet = univ.Sequence.tagSet.tagImplicitly(
        tag.Tag(tag.tagClassApplication, tag.tagFormatConstructed, 11)
    )


class ModifyDNRequest(univ.Sequence):
    tagSet = univ.Sequence.tagSet.tagImplicitly(
        tag.Tag(tag.tagClassApplication, tag.tagFormatConstructed, 12)
    )
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("entry", LDAPDN()),
        namedtype.NamedType("newrdn", RelativeLDAPDN()),
        namedtype.NamedType("deleteoldrdn", univ.Boolean()),
        namedtype.OptionalNamedType(
            "newSuperior",
            LDAPDN().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
            ),
        ),
    )


class ModifyDNResponse(LDAPResult):
    tagSet = univ.Sequence.tagSet.tagImplicitly(
        tag.Tag(tag.tagClassApplication, tag.tagFormatConstructed, 13)
    )


class CompareRequest(univ.Sequence):
    tagSet = univ.Sequence.tagSet.tagImplicitly(
        tag.Tag(tag.tagClassApplication, tag.tagFormatConstructed, 14)
    )
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("entry", LDAPDN()),
        namedtype.NamedType("ava", AttributeValueAssertion()),
    )


class CompareResponse(LDAPResult):
    tagSet = univ.Sequence.tagSet.tagImplicitly(
        tag.Tag(tag.tagClassApplication, tag.tagFormatConstructed, 15)
    )


class AbandonRequest(LDAPResult):
    tagSet = univ.Sequence.tagSet.tagImplicitly(
        tag.Tag(tag.tagClassApplication, tag.tagFormatConstructed, 16)
    )


class ExtendedRequest(univ.Sequence):
    tagSet = univ.Sequence.tagSet.tagImplicitly(
        tag.Tag(tag.tagClassApplication, tag.tagFormatConstructed, 23)
    )
    componentType = namedtype.NamedTypes(
        namedtype.NamedType(
            "requestName",
            LDAPOID().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
            ),
        ),
        namedtype.OptionalNamedType(
            "requestValue",
            univ.OctetString().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
            ),
        ),
    )


class ExtendedResponse(univ.Sequence):
    tagSet = univ.Sequence.tagSet.tagImplicitly(
        tag.Tag(tag.tagClassApplication, tag.tagFormatConstructed, 24)
    )
    componentType = namedtype.NamedTypes(
        namedtype.NamedType(
            "resultCode",
            univ.Enumerated(
                namedValues=namedval.NamedValues(
                    ("success", 0),
                    ("operationsError", 1),
                    ("protocolError", 2),
                    ("timeLimitExceeded", 3),
                    ("sizeLimitExceeded", 4),
                    ("compareFalse", 5),
                    ("compareTrue", 6),
                    ("authMethodNotSupported", 7),
                    ("strongAuthRequired", 8),
                    ("reserved-9", 9),
                    ("referral", 10),
                    ("adminLimitExceeded", 11),
                    ("unavailableCriticalExtension", 12),
                    ("confidentialityRequired", 13),
                    ("saslBindInProgress", 14),
                    ("noSuchAttribute", 16),
                    ("undefinedAttributeType", 17),
                    ("inappropriateMatching", 18),
                    ("constraintViolation", 19),
                    ("attributeOrValueExists", 20),
                    ("invalidAttributeSyntax", 21),
                    ("noSuchObject", 32),
                    ("aliasProblem", 33),
                    ("invalidDNSyntax", 34),
                    ("reserved-35", 35),
                    ("aliasDereferencingProblem", 36),
                    ("inappropriateAuthentication", 48),
                    ("invalidCredentials", 49),
                    ("insufficientAccessRights", 50),
                    ("busy", 51),
                    ("unavailable", 52),
                    ("unwillingToPerform", 53),
                    ("loopDetect", 54),
                    ("namingViolation", 64),
                    ("objectClassViolation", 65),
                    ("notAllowedOnNonLeaf", 66),
                    ("notAllowedOnRDN", 67),
                    ("entryAlreadyExists", 68),
                    ("objectClassModsProhibited", 69),
                    ("reserved-70", 70),
                    ("affectsMultipleDSAs", 71),
                    ("other", 80),
                    ("reserved-81", 81),
                    ("reserved-82", 82),
                    ("reserved-83", 83),
                    ("reserved-84", 84),
                    ("reserved-85", 85),
                    ("reserved-86", 86),
                    ("reserved-87", 87),
                    ("reserved-88", 88),
                    ("reserved-89", 89),
                    ("reserved-90", 90),
                )
            ),
        ),
        namedtype.NamedType("matchedDN", LDAPDN()),
        namedtype.NamedType("errorMessage", LDAPString()),
        namedtype.OptionalNamedType(
            "referral",
            Referral().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 3)
            ),
        ),
        namedtype.OptionalNamedType(
            "responseName",
            LDAPOID().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 10)
            ),
        ),
        namedtype.OptionalNamedType(
            "response",
            univ.OctetString().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 11)
            ),
        ),
    )


class MessageID(univ.Integer):
    subtypeSpec = univ.Integer.subtypeSpec + constraint.ValueRangeConstraint(0, maxInt)


class LDAPMessage(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("messageID", MessageID()),
        namedtype.NamedType(
            "protocolOp",
            univ.Choice(
                componentType=namedtype.NamedTypes(
                    namedtype.NamedType("bindRequest", BindRequest()),
                    namedtype.NamedType("bindResponse", BindResponse()),
                    namedtype.NamedType("unbindRequest", UnbindRequest()),
                    namedtype.NamedType("searchRequest", SearchRequest()),
                    namedtype.NamedType("searchResEntry", SearchResultEntry()),
                    namedtype.NamedType("searchResDone", SearchResultDone()),
                    namedtype.NamedType("searchResRef", SearchResultReference()),
                    namedtype.NamedType("modifyRequest", ModifyRequest()),
                    namedtype.NamedType("modifyResponse", ModifyResponse()),
                    namedtype.NamedType("addRequest", AddRequest()),
                    namedtype.NamedType("addResponse", AddResponse()),
                    namedtype.NamedType("delRequest", DelRequest()),
                    namedtype.NamedType("delResponse", DelResponse()),
                    namedtype.NamedType("modDNRequest", ModifyDNRequest()),
                    namedtype.NamedType("modDNResponse", ModifyDNResponse()),
                    namedtype.NamedType("compareRequest", CompareRequest()),
                    namedtype.NamedType("compareResponse", CompareResponse()),
                    namedtype.NamedType("abandonRequest", AbandonRequest()),
                    namedtype.NamedType("extendedReq", ExtendedRequest()),
                    namedtype.NamedType("extendedResp", ExtendedResponse()),
                )
            ),
        ),
        namedtype.OptionalNamedType(
            "controls",
            Controls().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 0)
            ),
        ),
    )



#!/usr/bin/env python
#
# This file is part of pyasn1-modules software.
#
# Copyright (c) 2005-2020, Ilya Etingof <etingof@gmail.com>
# License: https://www.pysnmp.com/pyasn1/license.html
#
# Read ASN.1/PEM CMP message on stdin, parse into
# plain text, then build substrate from it
#
import sys

from pyasn1 import debug
from pyasn1.codec.der import decoder, encoder

from pyasn1_modules import pem, rfc4210

if len(sys.argv) == 2 and sys.argv[1] == "-d":
    debug.setLogger(debug.Debug("all"))
elif len(sys.argv) != 1:
    print(
        """Usage:
$ cat cmp.pem | %s [-d]"""
        % sys.argv[0]
    )
    sys.exit(-1)

pkiMessage = rfc4210.PKIMessage()

substrate = pem.readBase64FromFile(sys.stdin)
if not substrate:
    sys.exit(0)

pkiMsg, rest = decoder.decode(substrate, asn1Spec=pkiMessage)

print(pkiMsg.prettyPrint())

assert encoder.encode(pkiMsg) == substrate, "CMP message recode fails"



#
# This file is part of pyasn1-modules software.
#
# Created by Russ Housley with assistance from asn1ate v.0.6.0.
#
# Copyright (c) 2019, Vigil Security, LLC
# License: https://www.pysnmp.com/pyasn1/license.html
#
# TEST Company Classification Policies
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc3114.txt
#

from pyasn1.type import char, namedval, univ

from pyasn1_modules import rfc5755

id_smime = univ.ObjectIdentifier(
    (
        1,
        2,
        840,
        113549,
        1,
        9,
        16,
    )
)

id_tsp = id_smime + (7,)

id_tsp_TEST_Amoco = id_tsp + (1,)


class Amoco_SecurityClassification(univ.Integer):
    namedValues = namedval.NamedValues(
        ("amoco-general", 6),
        ("amoco-confidential", 7),
        ("amoco-highly-confidential", 8),
    )


id_tsp_TEST_Caterpillar = id_tsp + (2,)


class Caterpillar_SecurityClassification(univ.Integer):
    namedValues = namedval.NamedValues(
        ("caterpillar-public", 6),
        ("caterpillar-green", 7),
        ("caterpillar-yellow", 8),
        ("caterpillar-red", 9),
    )


id_tsp_TEST_Whirlpool = id_tsp + (3,)


class Whirlpool_SecurityClassification(univ.Integer):
    namedValues = namedval.NamedValues(
        ("whirlpool-public", 6),
        ("whirlpool-internal", 7),
        ("whirlpool-confidential", 8),
    )


id_tsp_TEST_Whirlpool_Categories = id_tsp + (4,)


class SecurityCategoryValues(univ.SequenceOf):
    componentType = char.UTF8String()


# Example SecurityCategoryValues: "LAW DEPARTMENT USE ONLY"
# Example SecurityCategoryValues: "HUMAN RESOURCES USE ONLY"


# Also, the privacy mark in the security label can contain a string,
# such as: "ATTORNEY-CLIENT PRIVILEGED INFORMATION"


# Map of security category type OIDs to security category added
# to the ones that are in rfc5755.py

_securityCategoryMapUpdate = {
    id_tsp_TEST_Whirlpool_Categories: SecurityCategoryValues(),
}

rfc5755.securityCategoryMap.update(_securityCategoryMapUpdate)



#
# This file is part of pyasn1-modules software.
#
# Created by Russ Housley
# Copyright (c) 2019, Vigil Security, LLC
# License: https://www.pysnmp.com/pyasn1/license.html
#
import sys
import unittest

from pyasn1.codec.der.decoder import decode as der_decoder
from pyasn1.codec.der.encoder import encode as der_encoder

from pyasn1_modules import pem, rfc5280, rfc5917


class ClearanceSponsorTestCase(unittest.TestCase):
    cert_pem_text = """\
MIID1DCCA1qgAwIBAgIUUc1IQGJpeYQ0XwOS2ZmVEb3aeZ0wCgYIKoZIzj0EAwMw
ZjELMAkGA1UEBhMCVVMxCzAJBgNVBAgTAlZBMRAwDgYDVQQHEwdIZXJuZG9uMRAw
DgYDVQQKEwdFeGFtcGxlMQwwCgYDVQQLEwNQQ0ExGDAWBgNVBAMTD3BjYS5leGFt
cGxlLmNvbTAeFw0xOTExMDUyMjIwNDZaFw0yMDExMDQyMjIwNDZaMIGSMQswCQYD
VQQGEwJVUzELMAkGA1UECBMCVkExEDAOBgNVBAcTB0hlcm5kb24xEDAOBgNVBAoT
B0V4YW1wbGUxIjAgBgNVBAsTGUh1bWFuIFJlc291cmNlIERlcGFydG1lbnQxDTAL
BgNVBAMTBEZyZWQxHzAdBgkqhkiG9w0BCQEWEGZyZWRAZXhhbXBsZS5jb20wdjAQ
BgcqhkjOPQIBBgUrgQQAIgNiAAQObFslQ2EBP0xlDJ3sRnsNaqm/woQgKpBispSx
XxK5bWUVpfnWsZnjLWhtDuPcu1BcBlM2g7gwL/aw8nUSIK3D8Ja9rTUQQXc3zxnk
cl8+8znNXHMGByRjPUH87C+TOrqjggGaMIIBljAdBgNVHQ4EFgQU5m711OqFDNGR
SWMOSzTXjpTLIFUwbwYDVR0jBGgwZoAUJuolDwsyICik11oKjf8t3L1/VGWhQ6RB
MD8xCzAJBgNVBAYTAlVTMQswCQYDVQQIDAJWQTEQMA4GA1UEBwwHSGVybmRvbjER
MA8GA1UECgwIQm9ndXMgQ0GCCQCls1QoG7BuRjAPBgNVHRMBAf8EBTADAQH/MAsG
A1UdDwQEAwIBhjBCBglghkgBhvhCAQ0ENRYzVGhpcyBjZXJ0aWZpY2F0ZSBjYW5u
b3QgYmUgdHJ1c3RlZCBmb3IgYW55IHB1cnBvc2UuMBUGA1UdIAQOMAwwCgYIKwYB
BQUHDQIwCgYDVR02BAMCAQIwfwYDVR0JBHgwdjBJBgNVBDcxQjBABgsqhkiG9w0B
CRAHAwMCBeAxLTArgAsqhkiG9w0BCRAHBIEcMBoMGEhVTUFOIFJFU09VUkNFUyBV
U0UgT05MWTApBglghkgBZQIBBUQxHAwaSHVtYW4gUmVzb3VyY2VzIERlcGFydG1l
bnQwCgYIKoZIzj0EAwMDaAAwZQIwVh/RypULFgPpAN0I7OvuMomRWnm/Hea3Hk8P
tTRz2Zai8iYat7oeAmGVgMhSXy2jAjEAuJW4l/CFatBy4W/lZ7gS3weBdBa5WEDI
FFMC7GjGtCeLtXYqWfBnRdK26dOaHLB2
"""

    def setUp(self):
        self.asn1Spec = rfc5280.Certificate()

    def testDerCodec(self):
        substrate = pem.readBase64fromText(self.cert_pem_text)
        asn1Object, rest = der_decoder(substrate, asn1Spec=self.asn1Spec)

        self.assertFalse(rest)
        self.assertTrue(asn1Object.prettyPrint())
        self.assertEqual(substrate, der_encoder(asn1Object))

        cs = rfc5917.DirectoryString()
        cs["utf8String"] = "Human Resources Department"
        encoded_cs = der_encoder(cs)

        clearance_sponsor_found = False

        for extn in asn1Object["tbsCertificate"]["extensions"]:
            if extn["extnID"] == rfc5280.id_ce_subjectDirectoryAttributes:

                self.assertIn(extn["extnID"], rfc5280.certificateExtensionsMap)

                ev, rest = der_decoder(
                    extn["extnValue"],
                    asn1Spec=rfc5280.certificateExtensionsMap[extn["extnID"]],
                )

                self.assertFalse(rest)
                self.assertTrue(ev.prettyPrint())
                self.assertEqual(extn["extnValue"], der_encoder(ev))

                for attr in ev:
                    if attr["type"] == rfc5917.id_clearanceSponsor:
                        self.assertEqual(encoded_cs, attr["values"][0])
                        clearance_sponsor_found = True

        self.assertTrue(clearance_sponsor_found)

    def testOpenTypes(self):
        substrate = pem.readBase64fromText(self.cert_pem_text)
        asn1Object, rest = der_decoder(
            substrate, asn1Spec=self.asn1Spec, decodeOpenTypes=True
        )

        self.assertFalse(rest)
        self.assertTrue(asn1Object.prettyPrint())
        self.assertEqual(substrate, der_encoder(asn1Object))

        clearance_sponsor_found = False

        for extn in asn1Object["tbsCertificate"]["extensions"]:
            if extn["extnID"] == rfc5280.id_ce_subjectDirectoryAttributes:
                self.assertIn(extn["extnID"], rfc5280.certificateExtensionsMap)

                ev, rest = der_decoder(
                    extn["extnValue"],
                    asn1Spec=rfc5280.certificateExtensionsMap[extn["extnID"]],
                    decodeOpenTypes=True,
                )

                self.assertFalse(rest)
                self.assertTrue(ev.prettyPrint())
                self.assertEqual(extn["extnValue"], der_encoder(ev))

                for attr in ev:
                    if attr["type"] == rfc5917.id_clearanceSponsor:
                        hrd = "Human Resources Department"

                        self.assertEqual(hrd, attr["values"][0]["utf8String"])

                        clearance_sponsor_found = True

        self.assertTrue(clearance_sponsor_found)


suite = unittest.TestLoader().loadTestsFromModule(sys.modules[__name__])

if __name__ == "__main__":
    unittest.TextTestRunner(verbosity=2).run(suite)



#
# This file is part of pyasn1-modules software.
#
# Created by Russ Housley with assistance from asn1ate v.0.6.0.
#
# Copyright (c) 2019, Vigil Security, LLC
# License: https://www.pysnmp.com/pyasn1/license.html
#
# CMS Symmetric Key Package Content Type
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc6031.txt
#

from pyasn1.type import (
    char,
    constraint,
    namedtype,
    namedval,
    opentype,
    tag,
    univ,
    useful,
)

from pyasn1_modules import rfc5652, rfc6019


def _OID(*components):
    output = []
    for x in tuple(components):
        if isinstance(x, univ.ObjectIdentifier):
            output.extend(list(x))
        else:
            output.append(int(x))
    return univ.ObjectIdentifier(output)


MAX = float("inf")

id_pskc = univ.ObjectIdentifier("1.2.840.113549.1.9.16.12")


# Symmetric Key Package Attributes

id_pskc_manufacturer = _OID(id_pskc, 1)


class at_pskc_manufacturer(char.UTF8String):
    pass


id_pskc_serialNo = _OID(id_pskc, 2)


class at_pskc_serialNo(char.UTF8String):
    pass


id_pskc_model = _OID(id_pskc, 3)


class at_pskc_model(char.UTF8String):
    pass


id_pskc_issueNo = _OID(id_pskc, 4)


class at_pskc_issueNo(char.UTF8String):
    pass


id_pskc_deviceBinding = _OID(id_pskc, 5)


class at_pskc_deviceBinding(char.UTF8String):
    pass


id_pskc_deviceStartDate = _OID(id_pskc, 6)


class at_pskc_deviceStartDate(useful.GeneralizedTime):
    pass


id_pskc_deviceExpiryDate = _OID(id_pskc, 7)


class at_pskc_deviceExpiryDate(useful.GeneralizedTime):
    pass


id_pskc_moduleId = _OID(id_pskc, 8)


class at_pskc_moduleId(char.UTF8String):
    pass


id_pskc_deviceUserId = _OID(id_pskc, 26)


class at_pskc_deviceUserId(char.UTF8String):
    pass


# Symmetric Key Attributes

id_pskc_keyId = _OID(id_pskc, 9)


class at_pskc_keyUserId(char.UTF8String):
    pass


id_pskc_algorithm = _OID(id_pskc, 10)


class at_pskc_algorithm(char.UTF8String):
    pass


id_pskc_issuer = _OID(id_pskc, 11)


class at_pskc_issuer(char.UTF8String):
    pass


id_pskc_keyProfileId = _OID(id_pskc, 12)


class at_pskc_keyProfileId(char.UTF8String):
    pass


id_pskc_keyReference = _OID(id_pskc, 13)


class at_pskc_keyReference(char.UTF8String):
    pass


id_pskc_friendlyName = _OID(id_pskc, 14)


class FriendlyName(univ.Sequence):
    pass


FriendlyName.componentType = namedtype.NamedTypes(
    namedtype.NamedType("friendlyName", char.UTF8String()),
    namedtype.OptionalNamedType("friendlyNameLangTag", char.UTF8String()),
)


class at_pskc_friendlyName(FriendlyName):
    pass


id_pskc_algorithmParameters = _OID(id_pskc, 15)


class Encoding(char.UTF8String):
    pass


Encoding.namedValues = namedval.NamedValues(
    ("dec", "DECIMAL"),
    ("hex", "HEXADECIMAL"),
    ("alpha", "ALPHANUMERIC"),
    ("b64", "BASE64"),
    ("bin", "BINARY"),
)

Encoding.subtypeSpec = constraint.SingleValueConstraint(
    "DECIMAL", "HEXADECIMAL", "ALPHANUMERIC", "BASE64", "BINARY"
)


class ChallengeFormat(univ.Sequence):
    pass


ChallengeFormat.componentType = namedtype.NamedTypes(
    namedtype.NamedType("encoding", Encoding()),
    namedtype.DefaultedNamedType("checkDigit", univ.Boolean().subtype(value=0)),
    namedtype.NamedType(
        "min",
        univ.Integer().subtype(subtypeSpec=constraint.ValueRangeConstraint(0, MAX)),
    ),
    namedtype.NamedType(
        "max",
        univ.Integer().subtype(subtypeSpec=constraint.ValueRangeConstraint(0, MAX)),
    ),
)


class ResponseFormat(univ.Sequence):
    pass


ResponseFormat.componentType = namedtype.NamedTypes(
    namedtype.NamedType("encoding", Encoding()),
    namedtype.NamedType(
        "length",
        univ.Integer().subtype(subtypeSpec=constraint.ValueRangeConstraint(0, MAX)),
    ),
    namedtype.DefaultedNamedType("checkDigit", univ.Boolean().subtype(value=0)),
)


class PSKCAlgorithmParameters(univ.Choice):
    pass


PSKCAlgorithmParameters.componentType = namedtype.NamedTypes(
    namedtype.NamedType("suite", char.UTF8String()),
    namedtype.NamedType(
        "challengeFormat",
        ChallengeFormat().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 0)
        ),
    ),
    namedtype.NamedType(
        "responseFormat",
        ResponseFormat().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 1)
        ),
    ),
)


class at_pskc_algorithmParameters(PSKCAlgorithmParameters):
    pass


id_pskc_counter = _OID(id_pskc, 16)


class at_pskc_counter(univ.Integer):
    pass


at_pskc_counter.subtypeSpec = constraint.ValueRangeConstraint(0, MAX)


id_pskc_time = _OID(id_pskc, 17)


class at_pskc_time(rfc6019.BinaryTime):
    pass


id_pskc_timeInterval = _OID(id_pskc, 18)


class at_pskc_timeInterval(univ.Integer):
    pass


at_pskc_timeInterval.subtypeSpec = constraint.ValueRangeConstraint(0, MAX)


id_pskc_timeDrift = _OID(id_pskc, 19)


class at_pskc_timeDrift(univ.Integer):
    pass


at_pskc_timeDrift.subtypeSpec = constraint.ValueRangeConstraint(0, MAX)


id_pskc_valueMAC = _OID(id_pskc, 20)


class ValueMac(univ.Sequence):
    pass


ValueMac.componentType = namedtype.NamedTypes(
    namedtype.NamedType("macAlgorithm", char.UTF8String()),
    namedtype.NamedType("mac", char.UTF8String()),
)


class at_pskc_valueMAC(ValueMac):
    pass


id_pskc_keyUserId = _OID(id_pskc, 27)


class at_pskc_keyId(char.UTF8String):
    pass


id_pskc_keyStartDate = _OID(id_pskc, 21)


class at_pskc_keyStartDate(useful.GeneralizedTime):
    pass


id_pskc_keyExpiryDate = _OID(id_pskc, 22)


class at_pskc_keyExpiryDate(useful.GeneralizedTime):
    pass


id_pskc_numberOfTransactions = _OID(id_pskc, 23)


class at_pskc_numberOfTransactions(univ.Integer):
    pass


at_pskc_numberOfTransactions.subtypeSpec = constraint.ValueRangeConstraint(0, MAX)


id_pskc_keyUsages = _OID(id_pskc, 24)


class PSKCKeyUsage(char.UTF8String):
    pass


PSKCKeyUsage.namedValues = namedval.NamedValues(
    ("otp", "OTP"),
    ("cr", "CR"),
    ("encrypt", "Encrypt"),
    ("integrity", "Integrity"),
    ("verify", "Verify"),
    ("unlock", "Unlock"),
    ("decrypt", "Decrypt"),
    ("keywrap", "KeyWrap"),
    ("unwrap", "Unwrap"),
    ("derive", "Derive"),
    ("generate", "Generate"),
)

PSKCKeyUsage.subtypeSpec = constraint.SingleValueConstraint(
    "OTP",
    "CR",
    "Encrypt",
    "Integrity",
    "Verify",
    "Unlock",
    "Decrypt",
    "KeyWrap",
    "Unwrap",
    "Derive",
    "Generate",
)


class PSKCKeyUsages(univ.SequenceOf):
    pass


PSKCKeyUsages.componentType = PSKCKeyUsage()


class at_pskc_keyUsage(PSKCKeyUsages):
    pass


id_pskc_pinPolicy = _OID(id_pskc, 25)


class PINUsageMode(char.UTF8String):
    pass


PINUsageMode.namedValues = namedval.NamedValues(
    ("local", "Local"),
    ("prepend", "Prepend"),
    ("append", "Append"),
    ("algorithmic", "Algorithmic"),
)

PINUsageMode.subtypeSpec = constraint.SingleValueConstraint(
    "Local", "Prepend", "Append", "Algorithmic"
)


class PINPolicy(univ.Sequence):
    pass


PINPolicy.componentType = namedtype.NamedTypes(
    namedtype.OptionalNamedType(
        "pinKeyId",
        char.UTF8String().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
        ),
    ),
    namedtype.NamedType(
        "pinUsageMode",
        PINUsageMode().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
        ),
    ),
    namedtype.OptionalNamedType(
        "maxFailedAttempts",
        univ.Integer()
        .subtype(subtypeSpec=constraint.ValueRangeConstraint(0, MAX))
        .subtype(implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 2)),
    ),
    namedtype.OptionalNamedType(
        "minLength",
        univ.Integer()
        .subtype(subtypeSpec=constraint.ValueRangeConstraint(0, MAX))
        .subtype(implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 3)),
    ),
    namedtype.OptionalNamedType(
        "maxLength",
        univ.Integer()
        .subtype(subtypeSpec=constraint.ValueRangeConstraint(0, MAX))
        .subtype(implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 4)),
    ),
    namedtype.OptionalNamedType(
        "pinEncoding",
        Encoding().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 5)
        ),
    ),
)


class at_pskc_pinPolicy(PINPolicy):
    pass


# Map of Symmetric Key Package Attribute OIDs to Attributes

sKeyPkgAttributesMap = {
    id_pskc_manufacturer: at_pskc_manufacturer(),
    id_pskc_serialNo: at_pskc_serialNo(),
    id_pskc_model: at_pskc_model(),
    id_pskc_issueNo: at_pskc_issueNo(),
    id_pskc_deviceBinding: at_pskc_deviceBinding(),
    id_pskc_deviceStartDate: at_pskc_deviceStartDate(),
    id_pskc_deviceExpiryDate: at_pskc_deviceExpiryDate(),
    id_pskc_moduleId: at_pskc_moduleId(),
    id_pskc_deviceUserId: at_pskc_deviceUserId(),
}


# Map of Symmetric Key Attribute OIDs to Attributes

sKeyAttributesMap = {
    id_pskc_keyId: at_pskc_keyId(),
    id_pskc_algorithm: at_pskc_algorithm(),
    id_pskc_issuer: at_pskc_issuer(),
    id_pskc_keyProfileId: at_pskc_keyProfileId(),
    id_pskc_keyReference: at_pskc_keyReference(),
    id_pskc_friendlyName: at_pskc_friendlyName(),
    id_pskc_algorithmParameters: at_pskc_algorithmParameters(),
    id_pskc_counter: at_pskc_counter(),
    id_pskc_time: at_pskc_time(),
    id_pskc_timeInterval: at_pskc_timeInterval(),
    id_pskc_timeDrift: at_pskc_timeDrift(),
    id_pskc_valueMAC: at_pskc_valueMAC(),
    id_pskc_keyUserId: at_pskc_keyUserId(),
    id_pskc_keyStartDate: at_pskc_keyStartDate(),
    id_pskc_keyExpiryDate: at_pskc_keyExpiryDate(),
    id_pskc_numberOfTransactions: at_pskc_numberOfTransactions(),
    id_pskc_keyUsages: at_pskc_keyUsage(),
    id_pskc_pinPolicy: at_pskc_pinPolicy(),
}


# This definition replaces Attribute() from rfc5652.py; it is the same except
# that opentype is added with sKeyPkgAttributesMap and sKeyAttributesMap


class AttributeType(univ.ObjectIdentifier):
    pass


class AttributeValue(univ.Any):
    pass


class SKeyAttribute(univ.Sequence):
    pass


SKeyAttribute.componentType = namedtype.NamedTypes(
    namedtype.NamedType("attrType", AttributeType()),
    namedtype.NamedType(
        "attrValues",
        univ.SetOf(componentType=AttributeValue()),
        openType=opentype.OpenType("attrType", sKeyAttributesMap),
    ),
)


class SKeyPkgAttribute(univ.Sequence):
    pass


SKeyPkgAttribute.componentType = namedtype.NamedTypes(
    namedtype.NamedType("attrType", AttributeType()),
    namedtype.NamedType(
        "attrValues",
        univ.SetOf(componentType=AttributeValue()),
        openType=opentype.OpenType("attrType", sKeyPkgAttributesMap),
    ),
)


# Symmetric Key Package Content Type

id_ct_KP_sKeyPackage = univ.ObjectIdentifier("1.2.840.113549.1.9.16.1.25")


class KeyPkgVersion(univ.Integer):
    pass


KeyPkgVersion.namedValues = namedval.NamedValues(("v1", 1))


class OneSymmetricKey(univ.Sequence):
    pass


OneSymmetricKey.componentType = namedtype.NamedTypes(
    namedtype.OptionalNamedType(
        "sKeyAttrs",
        univ.SequenceOf(componentType=SKeyAttribute()).subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, MAX)
        ),
    ),
    namedtype.OptionalNamedType("sKey", univ.OctetString()),
)

OneSymmetricKey.sizeSpec = univ.Sequence.sizeSpec + constraint.ValueSizeConstraint(1, 2)


class SymmetricKeys(univ.SequenceOf):
    pass


SymmetricKeys.componentType = OneSymmetricKey()
SymmetricKeys.subtypeSpec = constraint.ValueSizeConstraint(1, MAX)


class SymmetricKeyPackage(univ.Sequence):
    pass


SymmetricKeyPackage.componentType = namedtype.NamedTypes(
    namedtype.DefaultedNamedType("version", KeyPkgVersion().subtype(value="v1")),
    namedtype.OptionalNamedType(
        "sKeyPkgAttrs",
        univ.SequenceOf(componentType=SKeyPkgAttribute()).subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, MAX),
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0),
        ),
    ),
    namedtype.NamedType("sKeys", SymmetricKeys()),
)


# Map of Content Type OIDs to Content Types are
# added to the ones that are in rfc5652.py

_cmsContentTypesMapUpdate = {
    id_ct_KP_sKeyPackage: SymmetricKeyPackage(),
}

rfc5652.cmsContentTypesMap.update(_cmsContentTypesMapUpdate)



#
# This file is part of pyasn1-modules software.
#
# Copyright (c) 2019, Vigil Security, LLC
# License: https://www.pysnmp.com/pyasn1/license.html
#
import sys
import unittest

from pyasn1.codec.der.decoder import decode as der_decoder
from pyasn1.codec.der.encoder import encode as der_encoder

from pyasn1_modules import pem, rfc4108, rfc5652


class CMSFirmwareWrapperTestCase(unittest.TestCase):
    pem_text = """\
MIIEvAYJKoZIhvcNAQcCoIIErTCCBKkCAQExDTALBglghkgBZQMEAgEwggIVBgsq
hkiG9w0BCRABEKCCAgQEggIA3ntqPr5kDpx+//pgWGfHCH/Ht4pbenGwXv80txyE
Y0I2mT9BUGz8ILkbhD7Xz89pBS5KhEJpthxH8WREJtvS+wL4BqYLt23wjWoZy5Gt
5dPzWgaNlV/aQ5AdfAY9ljmnNYnK8D8r8ur7bQM4cKUdxry+QA0nqXHMAOSpx4Um
8impCc0BICXaFfL3zBrNxyPubbFO9ofbYOAWaNmmIAhzthXf12vDrLostIqmYrP4
LMRCjTr4LeYaVrAWfKtbUbByN6IuBef3Qt5cJaChr74udz3JvbYFsUvCpl64kpRq
g2CT6R+xE4trO/pViJlI15dvJVz04BBYQ2jQsutJwChi97/DDcjIv03VBmrwRE0k
RJNFP9vpDM8CxJIqcobC5Kuv8b0GqGfGl6ouuQKEVMfBcrupgjk3oc3KL1iVdSr1
+74amb1vDtTMWNm6vWRqh+Kk17NGEi2mNvYkkZUTIHNGH7OgiDclFU8dSMZd1fun
/D9dmiFiErDB3Fzr4+8Qz0aKedNE/1uvM+dhu9qjuRdkDzZ4S7txTfk6y9pG9iyk
aEeTV2kElKXblgi+Cf0Ut4f5he8rt6jveHdMo9X36YiUQVvevj2cgN7lFivEnFYV
QY0xugpP7lvEFDfsi2+0ozgP8EKOLYaCUKpuvttlYJ+vdtUFEijizEZ4cx02RsXm
EesxggJ6MIICdgIBA4AUnutnybladNRNLxY5ZoDoAbXLpJwwCwYJYIZIAWUDBAIB
oIG8MBoGCSqGSIb3DQEJAzENBgsqhkiG9w0BCRABEDArBgsqhkiG9w0BCRACJDEc
MBoGCysGAQQBjb9BAQEqBgsrBgEEAY2/QQEBMDAvBgkqhkiG9w0BCQQxIgQgAJfv
uasB4P6WDLOkOyvj33YPgZW4olHbidzyh1EKP9YwQAYLKoZIhvcNAQkQAikxMTAv
MAsGCWCGSAFlAwQCAQQgAJfvuasB4P6WDLOkOyvj33YPgZW4olHbidzyh1EKP9Yw
CwYJKoZIhvcNAQELBIIBgDivAlSLbMPPu+zV+pPcYpNp+A1mwVOytjMBzSo31kR/
qEu+hVrDknAOk9IdCaDvcz612CcfNT85/KzrYvWWxOP2woU/vZj253SnndALpfNN
n3/crJjF6hKgkjUwoXebI7kuj5WCh2q5lkd6xUa+jkCw+CINcN43thtS66UsVI4d
mv02EvsS2cxPY/508uaQZ6AYAacm667bgX8xEjbzACMOeMCuvKQXWAuh3DkNk+gV
xizHDw7xZxXgMGMAnJglAeBtd3Si5ztILw9U2gKUqFn/nOgy+eW63JuU/q31/Hgg
ZATjyBznSzneTZrw8/ePoSCj7E9vBeCTUkeFbVB2tJK1iYDMblp6HUuwgYuGKXy/
ZwKL3GvB11qg7ntdEyjdLq0xcVrht/K0d2dPo4iO4Ac7c1xbFMDAlWOt4FMPWh6O
iTh55YvT7hAJjTbB5ebgMA9QJnAczQPFnaIePnlFrkETd3YyLK4yHwnoIGo1GiW/
dsnhVtIdkPtfJIvcYteYJg==
"""

    def setUp(self):
        self.asn1Spec = rfc5652.ContentInfo()

    def testDerCodec(self):
        substrate = pem.readBase64fromText(self.pem_text)

        asn1Object, rest = der_decoder(substrate, asn1Spec=self.asn1Spec)

        self.assertFalse(rest)
        self.assertTrue(asn1Object.prettyPrint())
        self.assertEqual(substrate, der_encoder(asn1Object))

        self.assertEqual(rfc5652.id_signedData, asn1Object["contentType"])

        inner, rest = der_decoder(asn1Object["content"], asn1Spec=rfc5652.SignedData())

        self.assertEqual(
            rfc4108.id_ct_firmwarePackage, inner["encapContentInfo"]["eContentType"]
        )

        self.assertTrue(inner["encapContentInfo"]["eContent"])

        attribute_list = []

        for attr in inner["signerInfos"][0]["signedAttrs"]:
            attribute_list.append(attr["attrType"])
            if attr["attrType"] == rfc4108.id_aa_targetHardwareIDs:
                av, rest = der_decoder(
                    attr["attrValues"][0], asn1Spec=rfc4108.TargetHardwareIdentifiers()
                )
                self.assertEqual(2, len(av))

                for oid in av:
                    self.assertIn("1.3.6.1.4.1.221121.1.1.", oid.prettyPrint())

        self.assertIn(rfc5652.id_contentType, attribute_list)
        self.assertIn(rfc5652.id_messageDigest, attribute_list)
        self.assertIn(rfc4108.id_aa_targetHardwareIDs, attribute_list)
        self.assertIn(rfc4108.id_aa_fwPkgMessageDigest, attribute_list)

    def testOpenTypes(self):
        substrate = pem.readBase64fromText(self.pem_text)
        asn1Object, rest = der_decoder(
            substrate, asn1Spec=self.asn1Spec, decodeOpenTypes=True
        )

        self.assertFalse(rest)
        self.assertTrue(asn1Object.prettyPrint())
        self.assertEqual(substrate, der_encoder(asn1Object))

        self.assertEqual(asn1Object["contentType"], rfc5652.id_signedData)

        sd_eci = asn1Object["content"]["encapContentInfo"]

        self.assertEqual(sd_eci["eContentType"], rfc4108.id_ct_firmwarePackage)
        self.assertTrue(sd_eci["eContent"].hasValue())

        for attr in asn1Object["content"]["signerInfos"][0]["signedAttrs"]:
            self.assertIn(attr["attrType"], rfc5652.cmsAttributesMap)
            if attr["attrType"] == rfc4108.id_aa_targetHardwareIDs:
                for oid in attr["attrValues"][0]:
                    self.assertIn("1.3.6.1.4.1.221121.1.1.", oid.prettyPrint())


suite = unittest.TestLoader().loadTestsFromModule(sys.modules[__name__])

if __name__ == "__main__":
    result = unittest.TextTestRunner(verbosity=2).run(suite)
    sys.exit(not result.wasSuccessful())



#!/usr/bin/env python
#
# This file is part of pyasn1-modules software.
#
# Copyright (c) 2005-2020, Ilya Etingof <etingof@gmail.com>
# License: https://www.pysnmp.com/pyasn1/license.html
#
# Read  bunch of ASN.1/PEM plain/encrypted private keys in PKCS#8
# format on stdin, parse each into plain text, then build substrate from it
#
import sys

from pyasn1.codec.der import decoder, encoder

from pyasn1_modules import pem, rfc5208

if len(sys.argv) != 1:
    print(
        """Usage:
$ cat pkcs8key.pem | %s"""
        % sys.argv[0]
    )
    sys.exit(-1)

cnt = 0

while True:
    idx, substrate = pem.readPemBlocksFromFile(
        sys.stdin,
        ("-----BEGIN PRIVATE KEY-----", "-----END PRIVATE KEY-----"),
        (
            "-----BEGIN ENCRYPTED PRIVATE KEY-----",
            "-----END ENCRYPTED PRIVATE KEY-----",
        ),
    )
    if not substrate:
        break

    if idx == 0:
        asn1Spec = rfc5208.PrivateKeyInfo()
    elif idx == 1:
        asn1Spec = rfc5208.EncryptedPrivateKeyInfo()
    else:
        break

    key, rest = decoder.decode(substrate, asn1Spec=asn1Spec)

    if rest:
        substrate = substrate[: -len(rest)]

    print(key.prettyPrint())

    assert encoder.encode(key) == substrate, "pkcs8 recode fails"

    cnt += 1

print("*** %s PKCS#8 key(s) de/serialized" % cnt)



#
# This file is part of pyasn1-modules software.
#
# Created by Russ Housley with assistance from asn1ate v.0.6.0.
#
# Copyright (c) 2019, Vigil Security, LLC
# License: https://www.pysnmp.com/pyasn1/license.html
#
# An Internet Attribute Certificate Profile for Authorization
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc5755.txt
# https://www.rfc-editor.org/rfc/rfc5912.txt (see Section 13)
#

from pyasn1.type import (
    char,
    constraint,
    namedtype,
    namedval,
    opentype,
    tag,
    univ,
    useful,
)

from pyasn1_modules import rfc5280, rfc5652

MAX = float("inf")

# Map for Security Category type to value

securityCategoryMap = {}


# Imports from RFC 5652

ContentInfo = rfc5652.ContentInfo


# Imports from RFC 5280

AlgorithmIdentifier = rfc5280.AlgorithmIdentifier

Attribute = rfc5280.Attribute

AuthorityInfoAccessSyntax = rfc5280.AuthorityInfoAccessSyntax

AuthorityKeyIdentifier = rfc5280.AuthorityKeyIdentifier

CertificateSerialNumber = rfc5280.CertificateSerialNumber

CRLDistributionPoints = rfc5280.CRLDistributionPoints

Extensions = rfc5280.Extensions

Extension = rfc5280.Extension

GeneralNames = rfc5280.GeneralNames

GeneralName = rfc5280.GeneralName

UniqueIdentifier = rfc5280.UniqueIdentifier


# Object Identifier arcs

id_pkix = univ.ObjectIdentifier(
    (
        1,
        3,
        6,
        1,
        5,
        5,
        7,
    )
)

id_pe = id_pkix + (1,)

id_kp = id_pkix + (3,)

id_aca = id_pkix + (10,)

id_ad = id_pkix + (48,)

id_at = univ.ObjectIdentifier(
    (
        2,
        5,
        4,
    )
)

id_ce = univ.ObjectIdentifier(
    (
        2,
        5,
        29,
    )
)


# Attribute Certificate


class AttCertVersion(univ.Integer):
    namedValues = namedval.NamedValues(("v2", 1))


class IssuerSerial(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("issuer", GeneralNames()),
        namedtype.NamedType("serial", CertificateSerialNumber()),
        namedtype.OptionalNamedType("issuerUID", UniqueIdentifier()),
    )


class ObjectDigestInfo(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType(
            "digestedObjectType",
            univ.Enumerated(
                namedValues=namedval.NamedValues(
                    ("publicKey", 0), ("publicKeyCert", 1), ("otherObjectTypes", 2)
                )
            ),
        ),
        namedtype.OptionalNamedType("otherObjectTypeID", univ.ObjectIdentifier()),
        namedtype.NamedType("digestAlgorithm", AlgorithmIdentifier()),
        namedtype.NamedType("objectDigest", univ.BitString()),
    )


class Holder(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.OptionalNamedType(
            "baseCertificateID",
            IssuerSerial().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 0)
            ),
        ),
        namedtype.OptionalNamedType(
            "entityName",
            GeneralNames().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
            ),
        ),
        namedtype.OptionalNamedType(
            "objectDigestInfo",
            ObjectDigestInfo().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 2)
            ),
        ),
    )


class V2Form(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.OptionalNamedType("issuerName", GeneralNames()),
        namedtype.OptionalNamedType(
            "baseCertificateID",
            IssuerSerial().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 0)
            ),
        ),
        namedtype.OptionalNamedType(
            "objectDigestInfo",
            ObjectDigestInfo().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 1)
            ),
        ),
    )


class AttCertIssuer(univ.Choice):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("v1Form", GeneralNames()),
        namedtype.NamedType(
            "v2Form",
            V2Form().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 0)
            ),
        ),
    )


class AttCertValidityPeriod(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("notBeforeTime", useful.GeneralizedTime()),
        namedtype.NamedType("notAfterTime", useful.GeneralizedTime()),
    )


class AttributeCertificateInfo(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("version", AttCertVersion()),
        namedtype.NamedType("holder", Holder()),
        namedtype.NamedType("issuer", AttCertIssuer()),
        namedtype.NamedType("signature", AlgorithmIdentifier()),
        namedtype.NamedType("serialNumber", CertificateSerialNumber()),
        namedtype.NamedType("attrCertValidityPeriod", AttCertValidityPeriod()),
        namedtype.NamedType("attributes", univ.SequenceOf(componentType=Attribute())),
        namedtype.OptionalNamedType("issuerUniqueID", UniqueIdentifier()),
        namedtype.OptionalNamedType("extensions", Extensions()),
    )


class AttributeCertificate(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("acinfo", AttributeCertificateInfo()),
        namedtype.NamedType("signatureAlgorithm", AlgorithmIdentifier()),
        namedtype.NamedType("signatureValue", univ.BitString()),
    )


# Attribute Certificate Extensions

id_pe_ac_auditIdentity = id_pe + (4,)

id_ce_noRevAvail = id_ce + (56,)

id_ce_targetInformation = id_ce + (55,)


class TargetCert(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("targetCertificate", IssuerSerial()),
        namedtype.OptionalNamedType("targetName", GeneralName()),
        namedtype.OptionalNamedType("certDigestInfo", ObjectDigestInfo()),
    )


class Target(univ.Choice):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType(
            "targetName",
            GeneralName().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
            ),
        ),
        namedtype.NamedType(
            "targetGroup",
            GeneralName().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
            ),
        ),
        namedtype.NamedType(
            "targetCert",
            TargetCert().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 2)
            ),
        ),
    )


class Targets(univ.SequenceOf):
    componentType = Target()


id_pe_ac_proxying = id_pe + (10,)


class ProxyInfo(univ.SequenceOf):
    componentType = Targets()


id_pe_aaControls = id_pe + (6,)


class AttrSpec(univ.SequenceOf):
    componentType = univ.ObjectIdentifier()


class AAControls(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.OptionalNamedType(
            "pathLenConstraint",
            univ.Integer().subtype(subtypeSpec=constraint.ValueRangeConstraint(0, MAX)),
        ),
        namedtype.OptionalNamedType(
            "permittedAttrs",
            AttrSpec().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
            ),
        ),
        namedtype.OptionalNamedType(
            "excludedAttrs",
            AttrSpec().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
            ),
        ),
        namedtype.DefaultedNamedType(
            "permitUnSpecified", univ.Boolean().subtype(value=1)
        ),
    )


# Attribute Certificate Attributes

id_aca_authenticationInfo = id_aca + (1,)


id_aca_accessIdentity = id_aca + (2,)


class SvceAuthInfo(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("service", GeneralName()),
        namedtype.NamedType("ident", GeneralName()),
        namedtype.OptionalNamedType("authInfo", univ.OctetString()),
    )


id_aca_chargingIdentity = id_aca + (3,)


id_aca_group = id_aca + (4,)


class IetfAttrSyntax(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.OptionalNamedType(
            "policyAuthority",
            GeneralNames().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
            ),
        ),
        namedtype.NamedType(
            "values",
            univ.SequenceOf(
                componentType=univ.Choice(
                    componentType=namedtype.NamedTypes(
                        namedtype.NamedType("octets", univ.OctetString()),
                        namedtype.NamedType("oid", univ.ObjectIdentifier()),
                        namedtype.NamedType("string", char.UTF8String()),
                    )
                )
            ),
        ),
    )


id_at_role = id_at + (72,)


class RoleSyntax(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.OptionalNamedType(
            "roleAuthority",
            GeneralNames().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
            ),
        ),
        namedtype.NamedType(
            "roleName",
            GeneralName().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
            ),
        ),
    )


class ClassList(univ.BitString):
    namedValues = namedval.NamedValues(
        ("unmarked", 0),
        ("unclassified", 1),
        ("restricted", 2),
        ("confidential", 3),
        ("secret", 4),
        ("topSecret", 5),
    )


class SecurityCategory(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType(
            "type",
            univ.ObjectIdentifier().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
            ),
        ),
        namedtype.NamedType(
            "value",
            univ.Any().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
            ),
            openType=opentype.OpenType("type", securityCategoryMap),
        ),
    )


id_at_clearance = univ.ObjectIdentifier(
    (
        2,
        5,
        4,
        55,
    )
)


class Clearance(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("policyId", univ.ObjectIdentifier()),
        namedtype.DefaultedNamedType(
            "classList", ClassList().subtype(value="unclassified")
        ),
        namedtype.OptionalNamedType(
            "securityCategories", univ.SetOf(componentType=SecurityCategory())
        ),
    )


id_at_clearance_rfc3281 = univ.ObjectIdentifier(
    (
        2,
        5,
        1,
        5,
        55,
    )
)


class Clearance_rfc3281(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType(
            "policyId",
            univ.ObjectIdentifier().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
            ),
        ),
        namedtype.DefaultedNamedType(
            "classList",
            ClassList()
            .subtype(implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1))
            .subtype(value="unclassified"),
        ),
        namedtype.OptionalNamedType(
            "securityCategories",
            univ.SetOf(componentType=SecurityCategory()).subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 2)
            ),
        ),
    )


id_aca_encAttrs = id_aca + (6,)


class ACClearAttrs(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("acIssuer", GeneralName()),
        namedtype.NamedType("acSerial", univ.Integer()),
        namedtype.NamedType("attrs", univ.SequenceOf(componentType=Attribute())),
    )


# Map of Certificate Extension OIDs to Extensions added to the
# ones that are in rfc5280.py

_certificateExtensionsMapUpdate = {
    id_pe_ac_auditIdentity: univ.OctetString(),
    id_ce_noRevAvail: univ.Null(),
    id_ce_targetInformation: Targets(),
    id_pe_ac_proxying: ProxyInfo(),
    id_pe_aaControls: AAControls(),
}

rfc5280.certificateExtensionsMap.update(_certificateExtensionsMapUpdate)


# Map of AttributeType OIDs to AttributeValue added to the
# ones that are in rfc5280.py

_certificateAttributesMapUpdate = {
    id_aca_authenticationInfo: SvceAuthInfo(),
    id_aca_accessIdentity: SvceAuthInfo(),
    id_aca_chargingIdentity: IetfAttrSyntax(),
    id_aca_group: IetfAttrSyntax(),
    id_at_role: RoleSyntax(),
    id_at_clearance: Clearance(),
    id_at_clearance_rfc3281: Clearance_rfc3281(),
    id_aca_encAttrs: ContentInfo(),
}

rfc5280.certificateAttributesMap.update(_certificateAttributesMapUpdate)



#!/usr/bin/env python
#
# This file is part of pyasn1-modules software.
#
# Copyright (c) 2005-2020, Ilya Etingof <etingof@gmail.com>
# License: https://www.pysnmp.com/pyasn1/license.html
#
# Read X.509 CRL on stdin, print them pretty and encode back into
# original wire format.
# CRL can be generated with "openssl openssl ca -gencrl ..." commands.
#
import sys

from pyasn1.codec.der import decoder, encoder

from pyasn1_modules import pem, rfc2459

if len(sys.argv) != 1:
    print(
        """Usage:
$ cat crl.pem | %s"""
        % sys.argv[0]
    )
    sys.exit(-1)

asn1Spec = rfc2459.CertificateList()

cnt = 0

while True:
    idx, substrate = pem.readPemBlocksFromFile(
        sys.stdin, ("-----BEGIN X509 CRL-----", "-----END X509 CRL-----")
    )
    if not substrate:
        break

    key, rest = decoder.decode(substrate, asn1Spec=asn1Spec)

    if rest:
        substrate = substrate[: -len(rest)]

    print(key.prettyPrint())

    assert encoder.encode(key) == substrate, "pkcs8 recode fails"

    cnt += 1

print("*** %s CRL(s) re/serialized" % cnt)
